import os
import pytest

from stanza.models import mwt_expander
from stanza.models.mwt.character_classifier import CharacterClassifier
from stanza.models.mwt.data import DataLoader
from stanza.models.mwt.trainer import Trainer
from stanza.utils.conll import CoNLL

pytestmark = [pytest.mark.pipeline, pytest.mark.travis]

ENG_TRAIN = """
# text = Elena's motorcycle tour
1-2	Elena's	_	_	_	_	_	_	_	_
1	Elena	Elena	PROPN	NNP	Number=Sing	4	nmod:poss	4:nmod:poss	_
2	's	's	PART	POS	_	1	case	1:case	_
3	motorcycle	motorcycle	NOUN	NN	Number=Sing	4	compound	4:compound	_
4	tour	tour	NOUN	NN	Number=Sing	0	root	0:root	_


# text = women's reproductive health
1-2	women's	_	_	_	_	_	_	_	_
1	women	woman	NOUN	NNS	Number=Plur	4	nmod:poss	4:nmod:poss	_
2	's	's	PART	POS	_	1	case	1:case	_
3	reproductive	reproductive	ADJ	JJ	Degree=Pos	4	amod	4:amod	_
4	health	health	NOUN	NN	Number=Sing	0	root	0:root	SpaceAfter=No


# text = The Chernobyl Children's Project
1	The	the	DET	DT	Definite=Def|PronType=Art	3	det	3:det	_
2	Chernobyl	Chernobyl	PROPN	NNP	Number=Sing	3	compound	3:compound	_
3-4	Children's	_	_	_	_	_	_	_	_
3	Children	Children	PROPN	NNP	Number=Sing	5	nmod:poss	5:nmod:poss	_
4	's	's	PART	POS	_	3	case	3:case	_
5	Project	Project	PROPN	NNP	Number=Sing	0	root	0:root	_

""".lstrip()

ENG_DEV = """
# text = The Chernobyl Children's Project
1	The	the	DET	DT	Definite=Def|PronType=Art	3	det	3:det	_
2	Chernobyl	Chernobyl	PROPN	NNP	Number=Sing	3	compound	3:compound	_
3-4	Children's	_	_	_	_	_	_	_	_
3	Children	Children	PROPN	NNP	Number=Sing	5	nmod:poss	5:nmod:poss	_
4	's	's	PART	POS	_	3	case	3:case	_
5	Project	Project	PROPN	NNP	Number=Sing	0	root	0:root	_

""".lstrip()

def test_train(tmp_path):
    test_train = str(os.path.join(tmp_path, "en_test.train.conllu"))
    with open(test_train, "w") as fout:
        fout.write(ENG_TRAIN)

    test_dev = str(os.path.join(tmp_path, "en_test.dev.conllu"))
    with open(test_dev, "w") as fout:
        fout.write(ENG_DEV)

    test_output = str(os.path.join(tmp_path, "en_test.dev.pred.conllu"))
    model_name = "en_test_mwt.pt"

    args = [
        "--data_dir", str(tmp_path),
        "--train_file", test_train,
        "--eval_file", test_dev,
        "--gold_file", test_dev,
        "--lang", "en",
        "--shorthand", "en_test",
        "--output_file", test_output,
        "--save_dir", str(tmp_path),
        "--save_name", model_name,
        "--num_epoch", "10",
    ]

    mwt_expander.main(args=args)

    model = Trainer(model_file=os.path.join(tmp_path, model_name))
    assert model.model is not None
    assert isinstance(model.model, CharacterClassifier)

    doc = CoNLL.conll2doc(input_str=ENG_DEV)
    dataloader = DataLoader(doc, 10, model.args, vocab=model.vocab, evaluation=True, expand_unk_vocab=True)
    preds = []
    for i, batch in enumerate(dataloader.to_loader()):
        assert i == 0 # there should only be one batch
        preds += model.predict(batch, never_decode_unk=True, vocab=dataloader.vocab)
    assert len(preds) == 1
    # it is possible to make a version of the test where this happens almost every time
    # for example, running for 100 epochs makes the model succeed 30 times in a row
    # (never saw a failure)
    # but the one time that failure happened, it would be really annoying
    #assert preds[0] == "Children 's"



"""
Utility functions for dealing with NER tagging.
"""

import logging

from stanza.models.common.vocab import EMPTY

logger = logging.getLogger('stanza')

EMPTY_TAG = ('_', '-', '', None)
EMPTY_OR_O_TAG = tuple(list(EMPTY_TAG) + ['O'])

def is_basic_scheme(all_tags):
    """
    Check if a basic tagging scheme is used. Return True if so.

    Args:
        all_tags: a list of NER tags

    Returns:
        True if the tagging scheme does not use B-, I-, etc, otherwise False
    """
    for tag in all_tags:
        if len(tag) > 2 and tag[:2] in ('B-', 'I-', 'S-', 'E-', 'B_', 'I_', 'S_', 'E_'):
            return False
    return True


def is_bio_scheme(all_tags):
    """
    Check if BIO tagging scheme is used. Return True if so.

    Args:
        all_tags: a list of NER tags
    
    Returns:
        True if the tagging scheme is BIO, otherwise False
    """
    for tag in all_tags:
        if tag in EMPTY_OR_O_TAG:
            continue
        elif len(tag) > 2 and tag[:2] in ('B-', 'I-', 'B_', 'I_'):
            continue
        else:
            return False
    return True

def to_bio2(tags):
    """
    Convert the original tag sequence to BIO2 format. If the input is already in BIO2 format,
    the original input is returned.

    Args:
        tags: a list of tags in either BIO or BIO2 format
    
    Returns:
        new_tags: a list of tags in BIO2 format
    """
    new_tags = []
    for i, tag in enumerate(tags):
        if tag in EMPTY_OR_O_TAG:
            new_tags.append(tag)
        elif tag[0] == 'I':
            if i == 0 or tags[i-1] == 'O' or tags[i-1][1:] != tag[1:]:
                new_tags.append('B' + tag[1:])
            else:
                new_tags.append(tag)
        else:
            new_tags.append(tag)
    return new_tags

def basic_to_bio(tags):
    """
    Convert a basic tag sequence into a BIO sequence.
    You can compose this with bio2_to_bioes to convert to bioes

    Args:
        tags: a list of tags in basic (no B-, I-, etc) format

    Returns:
        new_tags: a list of tags in BIO format
    """
    new_tags = []
    for i, tag in enumerate(tags):
        if tag in EMPTY_OR_O_TAG:
            new_tags.append(tag)
        elif i == 0 or tags[i-1] == 'O' or tags[i-1] != tag:
            new_tags.append('B-' + tag)
        else:
            new_tags.append('I-' + tag)
    return new_tags


def bio2_to_bioes(tags):
    """
    Convert the BIO2 tag sequence into a BIOES sequence.

    Args:
        tags: a list of tags in BIO2 format

    Returns:
        new_tags: a list of tags in BIOES format
    """
    new_tags = []
    for i, tag in enumerate(tags):
        if tag in EMPTY_OR_O_TAG:
            new_tags.append(tag)
        else:
            if len(tag) < 2:
                raise Exception(f"Invalid BIO2 tag found: {tag}")
            else:
                if tag[:2] in ('I-', 'I_'): # convert to E- if next tag is not I-
                    if i+1 < len(tags) and tags[i+1][:2] in ('I-', 'I_'):
                        new_tags.append('I-' + tag[2:]) # compensate for underscores
                    else:
                        new_tags.append('E-' + tag[2:])
                elif tag[:2] in ('B-', 'B_'): # convert to S- if next tag is not I-
                    if i+1 < len(tags) and tags[i+1][:2] in ('I-', 'I_'):
                        new_tags.append('B-' + tag[2:])
                    else:
                        new_tags.append('S-' + tag[2:])
                else:
                    raise Exception(f"Invalid IOB tag found: {tag}")
    return new_tags

def normalize_empty_tags(sentences):
    """
    If any tags are None, _, -, or blank, turn them into EMPTY

    The input should be a list(sentence) of list(word) of tuple(text, list(tag))
    which is the typical format for the data at the time data.py is preprocessing the tags
    """
    new_sentences = [[(word[0], tuple(EMPTY if x in EMPTY_TAG else x for x in word[1])) for word in sentence]
                     for sentence in sentences]
    return new_sentences

def process_tags(sentences, scheme):
    """
    Convert tags in these sentences to bioes

    We allow empty tags ('_', '-', None), which will represent tags
    that do not get any gradient when training
    """
    all_words = []
    all_tags = []
    converted_tuples = False
    for sent_idx, sent in enumerate(sentences):
        words, tags = zip(*sent)
        all_words.append(words)
        # if we got one dimension tags w/o tuples or lists, make them tuples
        # but we also check that the format is consistent,
        # as otherwise the result being converted might be confusing
        if not converted_tuples and any(tag is None or isinstance(tag, str) for tag in tags):
            if sent_idx > 0:
                raise ValueError("Got a mix of tags and lists of tags.  First non-list was in sentence %d" % sent_idx)
            converted_tuples = True
        if converted_tuples:
            if not all(tag is None or isinstance(tag, str) for tag in tags):
                raise ValueError("Got a mix of tags and lists of tags.  First tag as a list was in sentence %d" % sent_idx)
            tags = [(tag,) for tag in tags]
        all_tags.append(tags)

    max_columns = max(len(x) for tags in all_tags for x in tags)
    for sent_idx, tags in enumerate(all_tags):
        if any(len(x) < max_columns for x in tags):
            raise ValueError("NER tags not uniform in length at sentence %d.  TODO: extend those columns with O" % sent_idx)

    all_convert_bio_to_bioes = []
    all_convert_basic_to_bioes = []

    for column_idx in range(max_columns):
        # check if tag conversion is needed for each column
        # we treat each column separately, although practically
        # speaking it would be pretty weird for a dataset to have BIO
        # in one column and basic in another, for example
        convert_bio_to_bioes = False
        convert_basic_to_bioes = False
        tag_column = [x[column_idx] for sent in all_tags for x in sent]
        is_bio = is_bio_scheme(tag_column)
        is_basic = not is_bio and is_basic_scheme(tag_column)
        if is_bio and scheme.lower() == 'bioes':
            convert_bio_to_bioes = True
            logger.debug("BIO tagging scheme found in input at column %d; converting into BIOES scheme..." % column_idx)
        elif is_basic and scheme.lower() == 'bioes':
            convert_basic_to_bioes = True
            logger.debug("Basic tagging scheme found in input at column %d; converting into BIOES scheme..." % column_idx)
        all_convert_bio_to_bioes.append(convert_bio_to_bioes)
        all_convert_basic_to_bioes.append(convert_basic_to_bioes)

    result = []
    for words, tags in zip(all_words, all_tags):
        # TODO: add a convert_basic_to_bio option as well
        # process tags
        # tags is a list of each column of tags for each word in this sentence
        # copy the tags to a list so we can edit them
        tags = [[x for x in sentence_tags] for sentence_tags in tags]
        for column_idx, (convert_bio_to_bioes, convert_basic_to_bioes) in enumerate(zip(all_convert_bio_to_bioes, all_convert_basic_to_bioes)):
            tag_column = [x[column_idx] for x in tags]
            if convert_basic_to_bioes:
                # if basic, convert tags -> bio -> bioes
                tag_column = bio2_to_bioes(basic_to_bio(tag_column))
            else:
                # first ensure BIO2 scheme
                tag_column = to_bio2(tag_column)
                # then convert to BIOES
                if convert_bio_to_bioes:
                    tag_column = bio2_to_bioes(tag_column)
            for tag_idx, tag in enumerate(tag_column):
                tags[tag_idx][column_idx] = tag
        result.append([(w,tuple(t)) for w,t in zip(words, tags)])

    if converted_tuples:
        result = [[(word[0], word[1][0]) for word in sentence] for sentence in result]
    return result


def decode_from_bioes(tags):
    """
    Decode from a sequence of BIOES tags, assuming default tag is 'O'.
    Args:
        tags: a list of BIOES tags
    
    Returns:
        A list of dict with start_idx, end_idx, and type values.
    """
    res = []
    ent_idxs = []
    cur_type = None

    def flush():
        if len(ent_idxs) > 0:
            res.append({
                'start': ent_idxs[0], 
                'end': ent_idxs[-1], 
                'type': cur_type})

    for idx, tag in enumerate(tags):
        if tag is None:
            tag = 'O'
        if tag == 'O':
            flush()
            ent_idxs = []
        elif tag.startswith('B-'): # start of new ent
            flush()
            ent_idxs = [idx]
            cur_type = tag[2:]
        elif tag.startswith('I-'): # continue last ent
            ent_idxs.append(idx)
            cur_type = tag[2:]
        elif tag.startswith('E-'): # end last ent
            ent_idxs.append(idx)
            cur_type = tag[2:]
            flush()
            ent_idxs = []
        elif tag.startswith('S-'): # start single word ent
            flush()
            ent_idxs = [idx]
            cur_type = tag[2:]
            flush()
            ent_idxs = []
    # flush after whole sentence
    flush()
    return res


def merge_tags(*sequences):
    """
    Merge multiple sequences of NER tags into one sequence

    Only O is replaced, and the earlier tags have precedence
    """
    tags = list(sequences[0])
    for sequence in sequences[1:]:
        idx = 0
        while idx < len(sequence):
            # skip empty tags in the later sequences
            if sequence[idx] == 'O':
                idx += 1
                continue

            # check for singletons.  copy if not O in the original
            if sequence[idx].startswith("S-"):
                if tags[idx] == 'O':
                    tags[idx] = sequence[idx]
                idx += 1
                continue

            # at this point, we know we have a B-... sequence
            if not sequence[idx].startswith("B-"):
                raise ValueError("Got unexpected tag sequence at idx {}: {}".format(idx, sequence))

            # take the block of tags which are B- through E-
            start_idx = idx
            end_idx = start_idx + 1
            while end_idx < len(sequence):
                if sequence[end_idx][2:] != sequence[start_idx][2:]:
                    raise ValueError("Unexpected tag sequence at idx {}: {}".format(end_idx, sequence))
                if sequence[end_idx].startswith("E-"):
                    break
                if not sequence[end_idx].startswith("I-"):
                    raise ValueError("Unexpected tag sequence at idx {}: {}".format(end_idx, sequence))
                end_idx += 1
            if end_idx == len(sequence):
                raise ValueError("Got a sequence with an unclosed tag: {}".format(sequence))
            end_idx = end_idx + 1

            # if all tags in the original are O, we can overwrite
            # otherwise, keep the originals
            if all(x == 'O' for x in tags[start_idx:end_idx]):
                tags[start_idx:end_idx] = sequence[start_idx:end_idx]
            idx = end_idx

    return tags



import glob
import os
import shutil
import tempfile

import pytest

import stanza
from stanza.models.common.foundation_cache import FoundationCache, load_charlm
from stanza.tests import TEST_MODELS_DIR

pytestmark = [pytest.mark.travis, pytest.mark.pipeline]

def test_charlm_cache():
    models_path = os.path.join(TEST_MODELS_DIR, "en", "backward_charlm", "*")
    models = glob.glob(models_path)
    # we expect at least one English model downloaded for the tests
    assert len(models) >= 1
    model_file = models[0]

    cache = FoundationCache()
    with tempfile.TemporaryDirectory(dir=".") as test_dir:
        temp_file = os.path.join(test_dir, "charlm.pt")
        shutil.copy2(model_file, temp_file)
        # this will work
        model = load_charlm(temp_file)

        # this will save the model
        model = cache.load_charlm(temp_file)

    # this should no longer work
    with pytest.raises(FileNotFoundError):
        model = load_charlm(temp_file)

    # it should remember the cached version
    model = cache.load_charlm(temp_file)



"""
SCARE is a dataset of German text with sentiment annotations.

http://romanklinger.de/scare/

To run the script, pass in the directory where scare was unpacked.  It
should have subdirectories scare_v1.0.0 and scare_v1.0.0_text

You need to fill out a license agreement to not redistribute the data
in order to get the data, but the process is not onerous.

Although it sounds interesting, there are unfortunately a lot of very
short items.  Not sure the long items will be enough
"""


import csv
import glob
import os
import sys

import stanza

from stanza.models.classifiers.data import SentimentDatum
import stanza.utils.datasets.sentiment.process_utils as process_utils

def get_scare_snippets(nlp, csv_dir_path, text_id_map, filename_pattern="*.csv"):
    """
    Read snippets from the given CSV directory
    """
    num_short_items = 0

    snippets = []
    csv_files = glob.glob(os.path.join(csv_dir_path, filename_pattern))
    for csv_filename in csv_files:
        with open(csv_filename, newline='') as fin:
            cin = csv.reader(fin, delimiter='\t', quotechar='"')
            lines = list(cin)

            for line in lines:
                ann_id, begin, end, sentiment = [line[i] for i in [1, 2, 3, 6]]
                begin = int(begin)
                end = int(end)
                if sentiment.lower() == 'unknown':
                    continue
                elif sentiment.lower() == 'positive':
                    sentiment = 2
                elif sentiment.lower() == 'neutral':
                    sentiment = 1
                elif sentiment.lower() == 'negative':
                    sentiment = 0
                else:
                    raise ValueError("Tell John he screwed up and this is why he can't have Mox Opal: {}".format(sentiment))
                if ann_id not in text_id_map:
                    print("Found snippet which can't be found: {}-{}".format(csv_filename, ann_id))
                    continue
                snippet = text_id_map[ann_id][begin:end]
                doc = nlp(snippet)
                text = [token.text for sentence in doc.sentences for token in sentence.tokens]
                num_tokens = sum(len(sentence.tokens) for sentence in doc.sentences)
                if num_tokens < 4:
                    num_short_items = num_short_items + 1
                snippets.append(SentimentDatum(sentiment, text))
    print("Number of short items: {}".format(num_short_items))
    return snippets


def main(in_directory, out_directory, short_name):
    os.makedirs(out_directory, exist_ok=True)

    input_path = os.path.join(in_directory, "scare_v1.0.0_text", "annotations", "*txt")
    text_files = glob.glob(input_path)
    if len(text_files) == 0:
        raise FileNotFoundError("Did not find any input files in %s" % input_path)
    else:
        print("Found %d input files in %s" % (len(text_files), input_path))
    text_id_map = {}
    for filename in text_files:
        with open(filename) as fin:
            for line in fin.readlines():
                line = line.strip()
                if not line:
                    continue
                key, value = line.split(maxsplit=1)
                if key in text_id_map:
                    raise ValueError("Duplicate key {}".format(key))
                text_id_map[key] = value

    print("Found %d total sentiment ratings" % len(text_id_map))
    nlp = stanza.Pipeline('de', processors='tokenize')
    snippets = get_scare_snippets(nlp, os.path.join(in_directory, "scare_v1.0.0", "annotations"), text_id_map)

    print(len(snippets))
    process_utils.write_list(os.path.join(out_directory, "%s.train.json" % short_name), snippets)

if __name__ == '__main__':
    in_directory = sys.argv[1]
    out_directory = sys.argv[2]
    short_name = sys.argv[3]

    main(in_directory, out_directory, short_name)




import argparse
import re

TEXT_RE = re.compile("^#\\s*text")
NEWPAR_RE = re.compile("^#\\s*newpar")
NEWDOC_RE = re.compile("^#\\s*newdoc")

MWT_RE = re.compile("^\\d+-(\\d+)\t")
WORD_RE = re.compile("^(\\d)+\t")

WORD_NEWPAR_RE = re.compile("NewPar=Yes")
SPACEAFTER_RE = re.compile("SpaceAfter=No")

def print_new_paragraph_if_needed(fout, start, newdoc, newpar, output_buffer):
    if not start and (newdoc or newpar):
        if output_buffer:
            fout.write(output_buffer)
            fout.write("\n")
        fout.write("\n")
        return ""
    return output_buffer

def print_lines_from_buffer(fout, output_buffer, max_len):
    while len(output_buffer) >= max_len:
        split_idx = None
        for idx in range(len(output_buffer)):
            if idx > max_len and split_idx is not None:
                break
            if output_buffer[idx].isspace():
                split_idx = idx
        if split_idx is not None:
            fout.write(output_buffer[:split_idx])
            fout.write("\n")
            output_buffer = output_buffer[split_idx+1:]
        else:
            fout.write(output_buffer)
            fout.write("\n")
            output_buffer = ""
    return output_buffer

def convert_text(conllu_file, output_file):
    with open(conllu_file, encoding="utf-8") as fin:
        lines = fin.readlines()

    with open(output_file, "w", encoding="utf-8") as fout:
        newpar = False
        newdoc = False
        start = True

        in_mwt = False
        mwt_last = None

        def print_and_reset(output_buffer, incoming_buffer):
            nonlocal start, newpar, newdoc, in_mwt

            output_buffer = print_new_paragraph_if_needed(fout, start, newdoc, newpar, output_buffer)
            output_buffer += incoming_buffer
            output_buffer = print_lines_from_buffer(fout, output_buffer, 80)
            start = False
            newpar = False
            newdoc = False
            in_mwt = False
            return output_buffer

        output_buffer = ""
        incoming_buffer = ""

        for line in lines:
            line = line.strip()

            if not line:
                output_buffer = print_and_reset(output_buffer, incoming_buffer)
                incoming_buffer = ""

            if TEXT_RE.match(line):
                # we ignore the #text and extract the text from the tokens
                continue

            if NEWPAR_RE.match(line):
                newpar = True
                continue

            if NEWDOC_RE.match(line):
                newdoc = True
                continue

            match = MWT_RE.match(line)
            if match:
                in_mwt = True
                mwt_last = int(match.group(1))
                pieces = line.split("\t")

                if WORD_NEWPAR_RE.search(pieces[9]):
                    output_buffer = print_and_reset(output_buffer, incoming_buffer)
                    incoming_buffer = ""
                    fout.write(output_buffer)
                    fout.write("\n\n")
                    output_buffer = ""

                incoming_buffer += pieces[1]
                if not SPACEAFTER_RE.search(pieces[9]):
                    incoming_buffer += " "
                continue

            match = WORD_RE.match(line)
            if match:
                pieces = line.split("\t")
                word_id = int(pieces[0])
                if in_mwt and word_id <= mwt_last:
                    continue
                in_mwt = False

                if WORD_NEWPAR_RE.search(pieces[9]):
                    output_buffer = print_and_reset(output_buffer, incoming_buffer)
                    incoming_buffer = ""
                    fout.write(output_buffer)
                    fout.write("\n\n")
                    output_buffer = ""

                incoming_buffer += pieces[1]
                if not SPACEAFTER_RE.search(pieces[9]):
                    incoming_buffer += " "
                continue
        if output_buffer != "":
            fout.write(output_buffer)
            fout.write("\n")

def main(args=None):
    parser = argparse.ArgumentParser()
    parser.add_argument('conllu_file', type=str, help="CoNLL-U file containing tokens and sentence breaks")
    parser.add_argument('output_file', type=str, help="Plaintext file containing the raw input")
    args = parser.parse_args(args)

    convert_text(args.conllu_file, args.output_file)


if __name__ == "__main__":
    main()



"""
Small test of loading the Arabic pipeline

The main goal is to check that nothing goes wrong with RtL languages,
but incidentally this would have caught a bug where the xpos tags
were split into individual pieces instead of reassembled as expected
"""

import pytest
import stanza

from stanza.tests import TEST_MODELS_DIR

pytestmark = pytest.mark.pipeline

def test_arabic_pos_pipeline():
    pipe = stanza.Pipeline(**{'processors': 'tokenize,pos', 'dir': TEST_MODELS_DIR, 'download_method': None, 'lang': 'ar'})
    text = "ولم يتم اعتقال احد بحسب المتحدث باسم الشرطة."

    doc = pipe(text)
    # the first token translates to "and not", seems common enough
    # that we should be able to rely on it having a stable MWT and tag

    assert len(doc.sentences) == 1
    assert doc.sentences[0].tokens[0].text == "ولم"
    assert doc.sentences[0].words[0].xpos == "C---------"
    assert doc.sentences[0].words[1].xpos == "F---------"



"""
Build a dataset mixed with IAHLT Hebrew and UD Coref

We find that the IAHLT dataset by itself, trained using Stanza 1.11
with xlm-roberta-large and a lora finetuning layer, gets 49.7 F1.
This is a bit lower than the value the IAHLT group originally had, as
they reported 52.  Interestingly, we find that mixing in the 1.3 UD
Coref improves results, getting 51.7 under the same parameters

This script runs the IAHLT conversion and the UD Coref conversion,
then combines the files into one big training file
"""

import json
import os
import shutil
import tempfile

from stanza.utils.datasets.coref import convert_hebrew_iahlt
from stanza.utils.datasets.coref import convert_udcoref
from stanza.utils.default_paths import get_default_paths

def main():
    paths = get_default_paths()
    coref_output_path = paths['COREF_DATA_DIR']
    with tempfile.TemporaryDirectory() as temp_dir_path:
        hebrew_filenames = convert_hebrew_iahlt.main(["--output_directory", temp_dir_path])
        udcoref_filenames = convert_udcoref.main(["--project", "gerrom", "--output_directory", temp_dir_path])

        with open(os.path.join(temp_dir_path, hebrew_filenames[0]), encoding="utf-8") as fin:
            hebrew_train = json.load(fin)
        udcoref_train_filename = os.path.join(temp_dir_path, udcoref_filenames[0])
        with open(udcoref_train_filename, encoding="utf-8") as fin:
            print("Reading extra udcoref json data from %s" % udcoref_train_filename)
            udcoref_train = json.load(fin)
        mixed_train = hebrew_train + udcoref_train
        with open(os.path.join(coref_output_path, "he_mixed.train.json"), "w", encoding="utf-8") as fout:
            json.dump(mixed_train, fout, indent=2, ensure_ascii=False))

        shutil.copyfile(os.path.join(temp_dir_path, hebrew_filenames[1]),
                        os.path.join(coref_output_path, "he_mixed.dev.json"))
        shutil.copyfile(os.path.join(temp_dir_path, hebrew_filenames[2]),
                        os.path.join(coref_output_path, "he_mixed.test.json"))

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: CoreNLP.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rCoreNLP.proto\x12\x19\x65\x64u.stanford.nlp.pipeline\"\xe1\x05\n\x08\x44ocument\x12\x0c\n\x04text\x18\x01 \x02(\t\x12\x35\n\x08sentence\x18\x02 \x03(\x0b\x32#.edu.stanford.nlp.pipeline.Sentence\x12\x39\n\ncorefChain\x18\x03 \x03(\x0b\x32%.edu.stanford.nlp.pipeline.CorefChain\x12\r\n\x05\x64ocID\x18\x04 \x01(\t\x12\x0f\n\x07\x64ocDate\x18\x07 \x01(\t\x12\x10\n\x08\x63\x61lendar\x18\x08 \x01(\x04\x12;\n\x11sentencelessToken\x18\x05 \x03(\x0b\x32 .edu.stanford.nlp.pipeline.Token\x12\x33\n\tcharacter\x18\n \x03(\x0b\x32 .edu.stanford.nlp.pipeline.Token\x12/\n\x05quote\x18\x06 \x03(\x0b\x32 .edu.stanford.nlp.pipeline.Quote\x12\x37\n\x08mentions\x18\t \x03(\x0b\x32%.edu.stanford.nlp.pipeline.NERMention\x12#\n\x1bhasEntityMentionsAnnotation\x18\r \x01(\x08\x12\x0e\n\x06xmlDoc\x18\x0b \x01(\x08\x12\x34\n\x08sections\x18\x0c \x03(\x0b\x32\".edu.stanford.nlp.pipeline.Section\x12<\n\x10mentionsForCoref\x18\x0e \x03(\x0b\x32\".edu.stanford.nlp.pipeline.Mention\x12!\n\x19hasCorefMentionAnnotation\x18\x0f \x01(\x08\x12\x1a\n\x12hasCorefAnnotation\x18\x10 \x01(\x08\x12+\n#corefMentionToEntityMentionMappings\x18\x11 \x03(\x05\x12+\n#entityMentionToCorefMentionMappings\x18\x12 \x03(\x05*\x05\x08\x64\x10\x80\x02\"\xf3\x0f\n\x08Sentence\x12/\n\x05token\x18\x01 \x03(\x0b\x32 .edu.stanford.nlp.pipeline.Token\x12\x18\n\x10tokenOffsetBegin\x18\x02 \x02(\r\x12\x16\n\x0etokenOffsetEnd\x18\x03 \x02(\r\x12\x15\n\rsentenceIndex\x18\x04 \x01(\r\x12\x1c\n\x14\x63haracterOffsetBegin\x18\x05 \x01(\r\x12\x1a\n\x12\x63haracterOffsetEnd\x18\x06 \x01(\r\x12\x37\n\tparseTree\x18\x07 \x01(\x0b\x32$.edu.stanford.nlp.pipeline.ParseTree\x12@\n\x12\x62inarizedParseTree\x18\x1f \x01(\x0b\x32$.edu.stanford.nlp.pipeline.ParseTree\x12@\n\x12\x61nnotatedParseTree\x18  \x01(\x0b\x32$.edu.stanford.nlp.pipeline.ParseTree\x12\x11\n\tsentiment\x18! \x01(\t\x12=\n\x0fkBestParseTrees\x18\" \x03(\x0b\x32$.edu.stanford.nlp.pipeline.ParseTree\x12\x45\n\x11\x62\x61sicDependencies\x18\x08 \x01(\x0b\x32*.edu.stanford.nlp.pipeline.DependencyGraph\x12I\n\x15\x63ollapsedDependencies\x18\t \x01(\x0b\x32*.edu.stanford.nlp.pipeline.DependencyGraph\x12T\n collapsedCCProcessedDependencies\x18\n \x01(\x0b\x32*.edu.stanford.nlp.pipeline.DependencyGraph\x12K\n\x17\x61lternativeDependencies\x18\r \x01(\x0b\x32*.edu.stanford.nlp.pipeline.DependencyGraph\x12?\n\x0copenieTriple\x18\x0e \x03(\x0b\x32).edu.stanford.nlp.pipeline.RelationTriple\x12<\n\tkbpTriple\x18\x10 \x03(\x0b\x32).edu.stanford.nlp.pipeline.RelationTriple\x12\x45\n\x10\x65ntailedSentence\x18\x0f \x03(\x0b\x32+.edu.stanford.nlp.pipeline.SentenceFragment\x12\x43\n\x0e\x65ntailedClause\x18# \x03(\x0b\x32+.edu.stanford.nlp.pipeline.SentenceFragment\x12H\n\x14\x65nhancedDependencies\x18\x11 \x01(\x0b\x32*.edu.stanford.nlp.pipeline.DependencyGraph\x12P\n\x1c\x65nhancedPlusPlusDependencies\x18\x12 \x01(\x0b\x32*.edu.stanford.nlp.pipeline.DependencyGraph\x12\x33\n\tcharacter\x18\x13 \x03(\x0b\x32 .edu.stanford.nlp.pipeline.Token\x12\x11\n\tparagraph\x18\x0b \x01(\r\x12\x0c\n\x04text\x18\x0c \x01(\t\x12\x12\n\nlineNumber\x18\x14 \x01(\r\x12\x1e\n\x16hasRelationAnnotations\x18\x33 \x01(\x08\x12\x31\n\x06\x65ntity\x18\x34 \x03(\x0b\x32!.edu.stanford.nlp.pipeline.Entity\x12\x35\n\x08relation\x18\x35 \x03(\x0b\x32#.edu.stanford.nlp.pipeline.Relation\x12$\n\x1chasNumerizedTokensAnnotation\x18\x36 \x01(\x08\x12\x37\n\x08mentions\x18\x37 \x03(\x0b\x32%.edu.stanford.nlp.pipeline.NERMention\x12<\n\x10mentionsForCoref\x18\x38 \x03(\x0b\x32\".edu.stanford.nlp.pipeline.Mention\x12\"\n\x1ahasCorefMentionsAnnotation\x18\x39 \x01(\x08\x12\x12\n\nsentenceID\x18: \x01(\t\x12\x13\n\x0bsectionDate\x18; \x01(\t\x12\x14\n\x0csectionIndex\x18< \x01(\r\x12\x13\n\x0bsectionName\x18= \x01(\t\x12\x15\n\rsectionAuthor\x18> \x01(\t\x12\r\n\x05\x64ocID\x18? \x01(\t\x12\x15\n\rsectionQuoted\x18@ \x01(\x08\x12#\n\x1bhasEntityMentionsAnnotation\x18\x41 \x01(\x08\x12\x1f\n\x17hasKBPTriplesAnnotation\x18\x44 \x01(\x08\x12\"\n\x1ahasOpenieTriplesAnnotation\x18\x45 \x01(\x08\x12\x14\n\x0c\x63hapterIndex\x18\x42 \x01(\r\x12\x16\n\x0eparagraphIndex\x18\x43 \x01(\r\x12=\n\x10\x65nhancedSentence\x18\x46 \x01(\x0b\x32#.edu.stanford.nlp.pipeline.Sentence\x12\x0f\n\x07speaker\x18G \x01(\t\x12\x13\n\x0bspeakerType\x18H \x01(\t*\x05\x08\x64\x10\x80\x02\"\xf6\x0c\n\x05Token\x12\x0c\n\x04word\x18\x01 \x01(\t\x12\x0b\n\x03pos\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\x12\x10\n\x08\x63\x61tegory\x18\x04 \x01(\t\x12\x0e\n\x06\x62\x65\x66ore\x18\x05 \x01(\t\x12\r\n\x05\x61\x66ter\x18\x06 \x01(\t\x12\x14\n\x0coriginalText\x18\x07 \x01(\t\x12\x0b\n\x03ner\x18\x08 \x01(\t\x12\x11\n\tcoarseNER\x18> \x01(\t\x12\x16\n\x0e\x66ineGrainedNER\x18? \x01(\t\x12\x15\n\rnerLabelProbs\x18\x42 \x03(\t\x12\x15\n\rnormalizedNER\x18\t \x01(\t\x12\r\n\x05lemma\x18\n \x01(\t\x12\x11\n\tbeginChar\x18\x0b \x01(\r\x12\x0f\n\x07\x65ndChar\x18\x0c \x01(\r\x12\x11\n\tutterance\x18\r \x01(\r\x12\x0f\n\x07speaker\x18\x0e \x01(\t\x12\x13\n\x0bspeakerType\x18M \x01(\t\x12\x12\n\nbeginIndex\x18\x0f \x01(\r\x12\x10\n\x08\x65ndIndex\x18\x10 \x01(\r\x12\x17\n\x0ftokenBeginIndex\x18\x11 \x01(\r\x12\x15\n\rtokenEndIndex\x18\x12 \x01(\r\x12\x34\n\ntimexValue\x18\x13 \x01(\x0b\x32 .edu.stanford.nlp.pipeline.Timex\x12\x15\n\rhasXmlContext\x18\x15 \x01(\x08\x12\x12\n\nxmlContext\x18\x16 \x03(\t\x12\x16\n\x0e\x63orefClusterID\x18\x17 \x01(\r\x12\x0e\n\x06\x61nswer\x18\x18 \x01(\t\x12\x15\n\rheadWordIndex\x18\x1a \x01(\r\x12\x35\n\x08operator\x18\x1b \x01(\x0b\x32#.edu.stanford.nlp.pipeline.Operator\x12\x35\n\x08polarity\x18\x1c \x01(\x0b\x32#.edu.stanford.nlp.pipeline.Polarity\x12\x14\n\x0cpolarity_dir\x18\' \x01(\t\x12-\n\x04span\x18\x1d \x01(\x0b\x32\x1f.edu.stanford.nlp.pipeline.Span\x12\x11\n\tsentiment\x18\x1e \x01(\t\x12\x16\n\x0equotationIndex\x18\x1f \x01(\x05\x12\x42\n\x0e\x63onllUFeatures\x18  \x01(\x0b\x32*.edu.stanford.nlp.pipeline.MapStringString\x12\x11\n\tcoarseTag\x18! \x01(\t\x12\x38\n\x0f\x63onllUTokenSpan\x18\" \x01(\x0b\x32\x1f.edu.stanford.nlp.pipeline.Span\x12\x12\n\nconllUMisc\x18# \x01(\t\x12G\n\x13\x63onllUSecondaryDeps\x18$ \x01(\x0b\x32*.edu.stanford.nlp.pipeline.MapStringString\x12\x17\n\x0fwikipediaEntity\x18% \x01(\t\x12\x11\n\tisNewline\x18& \x01(\x08\x12\x0e\n\x06gender\x18\x33 \x01(\t\x12\x10\n\x08trueCase\x18\x34 \x01(\t\x12\x14\n\x0ctrueCaseText\x18\x35 \x01(\t\x12\x13\n\x0b\x63hineseChar\x18\x36 \x01(\t\x12\x12\n\nchineseSeg\x18\x37 \x01(\t\x12\x16\n\x0e\x63hineseXMLChar\x18< \x01(\t\x12\x11\n\tarabicSeg\x18L \x01(\t\x12\x13\n\x0bsectionName\x18\x38 \x01(\t\x12\x15\n\rsectionAuthor\x18\x39 \x01(\t\x12\x13\n\x0bsectionDate\x18: \x01(\t\x12\x17\n\x0fsectionEndLabel\x18; \x01(\t\x12\x0e\n\x06parent\x18= \x01(\t\x12\x19\n\x11\x63orefMentionIndex\x18@ \x03(\r\x12\x1a\n\x12\x65ntityMentionIndex\x18\x41 \x01(\r\x12\r\n\x05isMWT\x18\x43 \x01(\x08\x12\x12\n\nisFirstMWT\x18\x44 \x01(\x08\x12\x0f\n\x07mwtText\x18\x45 \x01(\t\x12\x0f\n\x07mwtMisc\x18N \x01(\t\x12\x14\n\x0cnumericValue\x18\x46 \x01(\x04\x12\x13\n\x0bnumericType\x18G \x01(\t\x12\x1d\n\x15numericCompositeValue\x18H \x01(\x04\x12\x1c\n\x14numericCompositeType\x18I \x01(\t\x12\x1c\n\x14\x63odepointOffsetBegin\x18J \x01(\r\x12\x1a\n\x12\x63odepointOffsetEnd\x18K \x01(\r\x12\r\n\x05index\x18O \x01(\r\x12\x12\n\nemptyIndex\x18P \x01(\r*\x05\x08\x64\x10\x80\x02\"\xe4\x03\n\x05Quote\x12\x0c\n\x04text\x18\x01 \x01(\t\x12\r\n\x05\x62\x65gin\x18\x02 \x01(\r\x12\x0b\n\x03\x65nd\x18\x03 \x01(\r\x12\x15\n\rsentenceBegin\x18\x05 \x01(\r\x12\x13\n\x0bsentenceEnd\x18\x06 \x01(\r\x12\x12\n\ntokenBegin\x18\x07 \x01(\r\x12\x10\n\x08tokenEnd\x18\x08 \x01(\r\x12\r\n\x05\x64ocid\x18\t \x01(\t\x12\r\n\x05index\x18\n \x01(\r\x12\x0e\n\x06\x61uthor\x18\x0b \x01(\t\x12\x0f\n\x07mention\x18\x0c \x01(\t\x12\x14\n\x0cmentionBegin\x18\r \x01(\r\x12\x12\n\nmentionEnd\x18\x0e \x01(\r\x12\x13\n\x0bmentionType\x18\x0f \x01(\t\x12\x14\n\x0cmentionSieve\x18\x10 \x01(\t\x12\x0f\n\x07speaker\x18\x11 \x01(\t\x12\x14\n\x0cspeakerSieve\x18\x12 \x01(\t\x12\x18\n\x10\x63\x61nonicalMention\x18\x13 \x01(\t\x12\x1d\n\x15\x63\x61nonicalMentionBegin\x18\x14 \x01(\r\x12\x1b\n\x13\x63\x61nonicalMentionEnd\x18\x15 \x01(\r\x12N\n\x1a\x61ttributionDependencyGraph\x18\x16 \x01(\x0b\x32*.edu.stanford.nlp.pipeline.DependencyGraph\"\xc7\x01\n\tParseTree\x12\x33\n\x05\x63hild\x18\x01 \x03(\x0b\x32$.edu.stanford.nlp.pipeline.ParseTree\x12\r\n\x05value\x18\x02 \x01(\t\x12\x17\n\x0fyieldBeginIndex\x18\x03 \x01(\r\x12\x15\n\ryieldEndIndex\x18\x04 \x01(\r\x12\r\n\x05score\x18\x05 \x01(\x01\x12\x37\n\tsentiment\x18\x06 \x01(\x0e\x32$.edu.stanford.nlp.pipeline.Sentiment\"\x9b\x04\n\x0f\x44\x65pendencyGraph\x12=\n\x04node\x18\x01 \x03(\x0b\x32/.edu.stanford.nlp.pipeline.DependencyGraph.Node\x12=\n\x04\x65\x64ge\x18\x02 \x03(\x0b\x32/.edu.stanford.nlp.pipeline.DependencyGraph.Edge\x12\x10\n\x04root\x18\x03 \x03(\rB\x02\x10\x01\x12/\n\x05token\x18\x04 \x03(\x0b\x32 .edu.stanford.nlp.pipeline.Token\x12\x14\n\x08rootNode\x18\x05 \x03(\rB\x02\x10\x01\x1aX\n\x04Node\x12\x15\n\rsentenceIndex\x18\x01 \x02(\r\x12\r\n\x05index\x18\x02 \x02(\r\x12\x16\n\x0e\x63opyAnnotation\x18\x03 \x01(\r\x12\x12\n\nemptyIndex\x18\x04 \x01(\r\x1a\xd6\x01\n\x04\x45\x64ge\x12\x0e\n\x06source\x18\x01 \x02(\r\x12\x0e\n\x06target\x18\x02 \x02(\r\x12\x0b\n\x03\x64\x65p\x18\x03 \x01(\t\x12\x0f\n\x07isExtra\x18\x04 \x01(\x08\x12\x12\n\nsourceCopy\x18\x05 \x01(\r\x12\x12\n\ntargetCopy\x18\x06 \x01(\r\x12\x13\n\x0bsourceEmpty\x18\x08 \x01(\r\x12\x13\n\x0btargetEmpty\x18\t \x01(\r\x12>\n\x08language\x18\x07 \x01(\x0e\x32#.edu.stanford.nlp.pipeline.Language:\x07Unknown\"\xc6\x02\n\nCorefChain\x12\x0f\n\x07\x63hainID\x18\x01 \x02(\x05\x12\x43\n\x07mention\x18\x02 \x03(\x0b\x32\x32.edu.stanford.nlp.pipeline.CorefChain.CorefMention\x12\x16\n\x0erepresentative\x18\x03 \x02(\r\x1a\xc9\x01\n\x0c\x43orefMention\x12\x11\n\tmentionID\x18\x01 \x01(\x05\x12\x13\n\x0bmentionType\x18\x02 \x01(\t\x12\x0e\n\x06number\x18\x03 \x01(\t\x12\x0e\n\x06gender\x18\x04 \x01(\t\x12\x0f\n\x07\x61nimacy\x18\x05 \x01(\t\x12\x12\n\nbeginIndex\x18\x06 \x01(\r\x12\x10\n\x08\x65ndIndex\x18\x07 \x01(\r\x12\x11\n\theadIndex\x18\t \x01(\r\x12\x15\n\rsentenceIndex\x18\n \x01(\r\x12\x10\n\x08position\x18\x0b \x01(\r\"\xef\x08\n\x07Mention\x12\x11\n\tmentionID\x18\x01 \x01(\x05\x12\x13\n\x0bmentionType\x18\x02 \x01(\t\x12\x0e\n\x06number\x18\x03 \x01(\t\x12\x0e\n\x06gender\x18\x04 \x01(\t\x12\x0f\n\x07\x61nimacy\x18\x05 \x01(\t\x12\x0e\n\x06person\x18\x06 \x01(\t\x12\x12\n\nstartIndex\x18\x07 \x01(\r\x12\x10\n\x08\x65ndIndex\x18\t \x01(\r\x12\x11\n\theadIndex\x18\n \x01(\x05\x12\x12\n\nheadString\x18\x0b \x01(\t\x12\x11\n\tnerString\x18\x0c \x01(\t\x12\x13\n\x0boriginalRef\x18\r \x01(\x05\x12\x1a\n\x12goldCorefClusterID\x18\x0e \x01(\x05\x12\x16\n\x0e\x63orefClusterID\x18\x0f \x01(\x05\x12\x12\n\nmentionNum\x18\x10 \x01(\x05\x12\x0f\n\x07sentNum\x18\x11 \x01(\x05\x12\r\n\x05utter\x18\x12 \x01(\x05\x12\x11\n\tparagraph\x18\x13 \x01(\x05\x12\x11\n\tisSubject\x18\x14 \x01(\x08\x12\x16\n\x0eisDirectObject\x18\x15 \x01(\x08\x12\x18\n\x10isIndirectObject\x18\x16 \x01(\x08\x12\x1b\n\x13isPrepositionObject\x18\x17 \x01(\x08\x12\x0f\n\x07hasTwin\x18\x18 \x01(\x08\x12\x0f\n\x07generic\x18\x19 \x01(\x08\x12\x13\n\x0bisSingleton\x18\x1a \x01(\x08\x12\x1a\n\x12hasBasicDependency\x18\x1b \x01(\x08\x12\x1d\n\x15hasEnhancedDependency\x18\x1c \x01(\x08\x12\x1b\n\x13hasContextParseTree\x18\x1d \x01(\x08\x12?\n\x0fheadIndexedWord\x18\x1e \x01(\x0b\x32&.edu.stanford.nlp.pipeline.IndexedWord\x12=\n\rdependingVerb\x18\x1f \x01(\x0b\x32&.edu.stanford.nlp.pipeline.IndexedWord\x12\x38\n\x08headWord\x18  \x01(\x0b\x32&.edu.stanford.nlp.pipeline.IndexedWord\x12;\n\x0bspeakerInfo\x18! \x01(\x0b\x32&.edu.stanford.nlp.pipeline.SpeakerInfo\x12=\n\rsentenceWords\x18\x32 \x03(\x0b\x32&.edu.stanford.nlp.pipeline.IndexedWord\x12<\n\x0coriginalSpan\x18\x33 \x03(\x0b\x32&.edu.stanford.nlp.pipeline.IndexedWord\x12\x12\n\ndependents\x18\x34 \x03(\t\x12\x19\n\x11preprocessedTerms\x18\x35 \x03(\t\x12\x13\n\x0b\x61ppositions\x18\x36 \x03(\x05\x12\x1c\n\x14predicateNominatives\x18\x37 \x03(\x05\x12\x18\n\x10relativePronouns\x18\x38 \x03(\x05\x12\x13\n\x0blistMembers\x18\x39 \x03(\x05\x12\x15\n\rbelongToLists\x18: \x03(\x05\"X\n\x0bIndexedWord\x12\x13\n\x0bsentenceNum\x18\x01 \x01(\x05\x12\x12\n\ntokenIndex\x18\x02 \x01(\x05\x12\r\n\x05\x64ocID\x18\x03 \x01(\x05\x12\x11\n\tcopyCount\x18\x04 \x01(\r\"4\n\x0bSpeakerInfo\x12\x13\n\x0bspeakerName\x18\x01 \x01(\t\x12\x10\n\x08mentions\x18\x02 \x03(\x05\"\"\n\x04Span\x12\r\n\x05\x62\x65gin\x18\x01 \x02(\r\x12\x0b\n\x03\x65nd\x18\x02 \x02(\r\"w\n\x05Timex\x12\r\n\x05value\x18\x01 \x01(\t\x12\x10\n\x08\x61ltValue\x18\x02 \x01(\t\x12\x0c\n\x04text\x18\x03 \x01(\t\x12\x0c\n\x04type\x18\x04 \x01(\t\x12\x0b\n\x03tid\x18\x05 \x01(\t\x12\x12\n\nbeginPoint\x18\x06 \x01(\r\x12\x10\n\x08\x65ndPoint\x18\x07 \x01(\r\"\xdb\x01\n\x06\x45ntity\x12\x11\n\theadStart\x18\x06 \x01(\r\x12\x0f\n\x07headEnd\x18\x07 \x01(\r\x12\x13\n\x0bmentionType\x18\x08 \x01(\t\x12\x16\n\x0enormalizedName\x18\t \x01(\t\x12\x16\n\x0eheadTokenIndex\x18\n \x01(\r\x12\x0f\n\x07\x63orefID\x18\x0b \x01(\t\x12\x10\n\x08objectID\x18\x01 \x01(\t\x12\x13\n\x0b\x65xtentStart\x18\x02 \x01(\r\x12\x11\n\textentEnd\x18\x03 \x01(\r\x12\x0c\n\x04type\x18\x04 \x01(\t\x12\x0f\n\x07subtype\x18\x05 \x01(\t\"\xb7\x01\n\x08Relation\x12\x0f\n\x07\x61rgName\x18\x06 \x03(\t\x12.\n\x03\x61rg\x18\x07 \x03(\x0b\x32!.edu.stanford.nlp.pipeline.Entity\x12\x11\n\tsignature\x18\x08 \x01(\t\x12\x10\n\x08objectID\x18\x01 \x01(\t\x12\x13\n\x0b\x65xtentStart\x18\x02 \x01(\r\x12\x11\n\textentEnd\x18\x03 \x01(\r\x12\x0c\n\x04type\x18\x04 \x01(\t\x12\x0f\n\x07subtype\x18\x05 \x01(\t\"\xb2\x01\n\x08Operator\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x1b\n\x13quantifierSpanBegin\x18\x02 \x02(\x05\x12\x19\n\x11quantifierSpanEnd\x18\x03 \x02(\x05\x12\x18\n\x10subjectSpanBegin\x18\x04 \x02(\x05\x12\x16\n\x0esubjectSpanEnd\x18\x05 \x02(\x05\x12\x17\n\x0fobjectSpanBegin\x18\x06 \x02(\x05\x12\x15\n\robjectSpanEnd\x18\x07 \x02(\x05\"\xa9\x04\n\x08Polarity\x12K\n\x12projectEquivalence\x18\x01 \x02(\x0e\x32/.edu.stanford.nlp.pipeline.NaturalLogicRelation\x12Q\n\x18projectForwardEntailment\x18\x02 \x02(\x0e\x32/.edu.stanford.nlp.pipeline.NaturalLogicRelation\x12Q\n\x18projectReverseEntailment\x18\x03 \x02(\x0e\x32/.edu.stanford.nlp.pipeline.NaturalLogicRelation\x12H\n\x0fprojectNegation\x18\x04 \x02(\x0e\x32/.edu.stanford.nlp.pipeline.NaturalLogicRelation\x12K\n\x12projectAlternation\x18\x05 \x02(\x0e\x32/.edu.stanford.nlp.pipeline.NaturalLogicRelation\x12\x45\n\x0cprojectCover\x18\x06 \x02(\x0e\x32/.edu.stanford.nlp.pipeline.NaturalLogicRelation\x12L\n\x13projectIndependence\x18\x07 \x02(\x0e\x32/.edu.stanford.nlp.pipeline.NaturalLogicRelation\"\xdd\x02\n\nNERMention\x12\x15\n\rsentenceIndex\x18\x01 \x01(\r\x12%\n\x1dtokenStartInSentenceInclusive\x18\x02 \x02(\r\x12#\n\x1btokenEndInSentenceExclusive\x18\x03 \x02(\r\x12\x0b\n\x03ner\x18\x04 \x02(\t\x12\x15\n\rnormalizedNER\x18\x05 \x01(\t\x12\x12\n\nentityType\x18\x06 \x01(\t\x12/\n\x05timex\x18\x07 \x01(\x0b\x32 .edu.stanford.nlp.pipeline.Timex\x12\x17\n\x0fwikipediaEntity\x18\x08 \x01(\t\x12\x0e\n\x06gender\x18\t \x01(\t\x12\x1a\n\x12\x65ntityMentionIndex\x18\n \x01(\r\x12#\n\x1b\x63\x61nonicalEntityMentionIndex\x18\x0b \x01(\r\x12\x19\n\x11\x65ntityMentionText\x18\x0c \x01(\t\"Y\n\x10SentenceFragment\x12\x12\n\ntokenIndex\x18\x01 \x03(\r\x12\x0c\n\x04root\x18\x02 \x01(\r\x12\x14\n\x0c\x61ssumedTruth\x18\x03 \x01(\x08\x12\r\n\x05score\x18\x04 \x01(\x01\":\n\rTokenLocation\x12\x15\n\rsentenceIndex\x18\x01 \x01(\r\x12\x12\n\ntokenIndex\x18\x02 \x01(\r\"\x9a\x03\n\x0eRelationTriple\x12\x0f\n\x07subject\x18\x01 \x01(\t\x12\x10\n\x08relation\x18\x02 \x01(\t\x12\x0e\n\x06object\x18\x03 \x01(\t\x12\x12\n\nconfidence\x18\x04 \x01(\x01\x12?\n\rsubjectTokens\x18\r \x03(\x0b\x32(.edu.stanford.nlp.pipeline.TokenLocation\x12@\n\x0erelationTokens\x18\x0e \x03(\x0b\x32(.edu.stanford.nlp.pipeline.TokenLocation\x12>\n\x0cobjectTokens\x18\x0f \x03(\x0b\x32(.edu.stanford.nlp.pipeline.TokenLocation\x12\x38\n\x04tree\x18\x08 \x01(\x0b\x32*.edu.stanford.nlp.pipeline.DependencyGraph\x12\x0e\n\x06istmod\x18\t \x01(\x08\x12\x10\n\x08prefixBe\x18\n \x01(\x08\x12\x10\n\x08suffixBe\x18\x0b \x01(\x08\x12\x10\n\x08suffixOf\x18\x0c \x01(\x08\"-\n\x0fMapStringString\x12\x0b\n\x03key\x18\x01 \x03(\t\x12\r\n\x05value\x18\x02 \x03(\t\"*\n\x0cMapIntString\x12\x0b\n\x03key\x18\x01 \x03(\r\x12\r\n\x05value\x18\x02 \x03(\t\"\xfc\x01\n\x07Section\x12\x11\n\tcharBegin\x18\x01 \x02(\r\x12\x0f\n\x07\x63harEnd\x18\x02 \x02(\r\x12\x0e\n\x06\x61uthor\x18\x03 \x01(\t\x12\x17\n\x0fsentenceIndexes\x18\x04 \x03(\r\x12\x10\n\x08\x64\x61tetime\x18\x05 \x01(\t\x12\x30\n\x06quotes\x18\x06 \x03(\x0b\x32 .edu.stanford.nlp.pipeline.Quote\x12\x17\n\x0f\x61uthorCharBegin\x18\x07 \x01(\r\x12\x15\n\rauthorCharEnd\x18\x08 \x01(\r\x12\x30\n\x06xmlTag\x18\t \x02(\x0b\x32 .edu.stanford.nlp.pipeline.Token\"\xe4\x01\n\x0eSemgrexRequest\x12\x0f\n\x07semgrex\x18\x01 \x03(\t\x12\x45\n\x05query\x18\x02 \x03(\x0b\x32\x36.edu.stanford.nlp.pipeline.SemgrexRequest.Dependencies\x1az\n\x0c\x44\x65pendencies\x12/\n\x05token\x18\x01 \x03(\x0b\x32 .edu.stanford.nlp.pipeline.Token\x12\x39\n\x05graph\x18\x02 \x02(\x0b\x32*.edu.stanford.nlp.pipeline.DependencyGraph\"\xfb\x05\n\x0fSemgrexResponse\x12\x46\n\x06result\x18\x01 \x03(\x0b\x32\x36.edu.stanford.nlp.pipeline.SemgrexResponse.GraphResult\x1a-\n\tNamedNode\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x12\n\nmatchIndex\x18\x02 \x02(\x05\x1a+\n\rNamedRelation\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x0c\n\x04reln\x18\x02 \x02(\t\x1a\x80\x01\n\tNamedEdge\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x0e\n\x06source\x18\x02 \x02(\x05\x12\x0e\n\x06target\x18\x03 \x02(\x05\x12\x0c\n\x04reln\x18\x04 \x01(\t\x12\x0f\n\x07isExtra\x18\x05 \x01(\x08\x12\x12\n\nsourceCopy\x18\x06 \x01(\r\x12\x12\n\ntargetCopy\x18\x07 \x01(\r\x1a\x95\x02\n\x05Match\x12\x12\n\nmatchIndex\x18\x01 \x02(\x05\x12\x42\n\x04node\x18\x02 \x03(\x0b\x32\x34.edu.stanford.nlp.pipeline.SemgrexResponse.NamedNode\x12\x46\n\x04reln\x18\x03 \x03(\x0b\x32\x38.edu.stanford.nlp.pipeline.SemgrexResponse.NamedRelation\x12\x42\n\x04\x65\x64ge\x18\x06 \x03(\x0b\x32\x34.edu.stanford.nlp.pipeline.SemgrexResponse.NamedEdge\x12\x12\n\ngraphIndex\x18\x04 \x01(\x05\x12\x14\n\x0csemgrexIndex\x18\x05 \x01(\x05\x1aP\n\rSemgrexResult\x12?\n\x05match\x18\x01 \x03(\x0b\x32\x30.edu.stanford.nlp.pipeline.SemgrexResponse.Match\x1aW\n\x0bGraphResult\x12H\n\x06result\x18\x01 \x03(\x0b\x32\x38.edu.stanford.nlp.pipeline.SemgrexResponse.SemgrexResult\"\xf0\x01\n\x0fSsurgeonRequest\x12\x45\n\x08ssurgeon\x18\x01 \x03(\x0b\x32\x33.edu.stanford.nlp.pipeline.SsurgeonRequest.Ssurgeon\x12\x39\n\x05graph\x18\x02 \x03(\x0b\x32*.edu.stanford.nlp.pipeline.DependencyGraph\x1a[\n\x08Ssurgeon\x12\x0f\n\x07semgrex\x18\x01 \x01(\t\x12\x11\n\toperation\x18\x02 \x03(\t\x12\n\n\x02id\x18\x03 \x01(\t\x12\r\n\x05notes\x18\x04 \x01(\t\x12\x10\n\x08language\x18\x05 \x01(\t\"\xbc\x01\n\x10SsurgeonResponse\x12J\n\x06result\x18\x01 \x03(\x0b\x32:.edu.stanford.nlp.pipeline.SsurgeonResponse.SsurgeonResult\x1a\\\n\x0eSsurgeonResult\x12\x39\n\x05graph\x18\x01 \x01(\x0b\x32*.edu.stanford.nlp.pipeline.DependencyGraph\x12\x0f\n\x07\x63hanged\x18\x02 \x01(\x08\"W\n\x12TokensRegexRequest\x12\x30\n\x03\x64oc\x18\x01 \x02(\x0b\x32#.edu.stanford.nlp.pipeline.Document\x12\x0f\n\x07pattern\x18\x02 \x03(\t\"\xa7\x03\n\x13TokensRegexResponse\x12J\n\x05match\x18\x01 \x03(\x0b\x32;.edu.stanford.nlp.pipeline.TokensRegexResponse.PatternMatch\x1a\x39\n\rMatchLocation\x12\x0c\n\x04text\x18\x01 \x01(\t\x12\r\n\x05\x62\x65gin\x18\x02 \x01(\x05\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x05\x1a\xb3\x01\n\x05Match\x12\x10\n\x08sentence\x18\x01 \x02(\x05\x12K\n\x05match\x18\x02 \x02(\x0b\x32<.edu.stanford.nlp.pipeline.TokensRegexResponse.MatchLocation\x12K\n\x05group\x18\x03 \x03(\x0b\x32<.edu.stanford.nlp.pipeline.TokensRegexResponse.MatchLocation\x1aS\n\x0cPatternMatch\x12\x43\n\x05match\x18\x01 \x03(\x0b\x32\x34.edu.stanford.nlp.pipeline.TokensRegexResponse.Match\"\xae\x01\n\x19\x44\x65pendencyEnhancerRequest\x12\x35\n\x08\x64ocument\x18\x01 \x02(\x0b\x32#.edu.stanford.nlp.pipeline.Document\x12\x37\n\x08language\x18\x02 \x01(\x0e\x32#.edu.stanford.nlp.pipeline.LanguageH\x00\x12\x1a\n\x10relativePronouns\x18\x03 \x01(\tH\x00\x42\x05\n\x03ref\"\xb4\x01\n\x12\x46lattenedParseTree\x12\x41\n\x05nodes\x18\x01 \x03(\x0b\x32\x32.edu.stanford.nlp.pipeline.FlattenedParseTree.Node\x1a[\n\x04Node\x12\x12\n\x08openNode\x18\x01 \x01(\x08H\x00\x12\x13\n\tcloseNode\x18\x02 \x01(\x08H\x00\x12\x0f\n\x05value\x18\x03 \x01(\tH\x00\x12\r\n\x05score\x18\x04 \x01(\x01\x42\n\n\x08\x63ontents\"\xf6\x01\n\x15\x45valuateParserRequest\x12N\n\x08treebank\x18\x01 \x03(\x0b\x32<.edu.stanford.nlp.pipeline.EvaluateParserRequest.ParseResult\x1a\x8c\x01\n\x0bParseResult\x12;\n\x04gold\x18\x01 \x02(\x0b\x32-.edu.stanford.nlp.pipeline.FlattenedParseTree\x12@\n\tpredicted\x18\x02 \x03(\x0b\x32-.edu.stanford.nlp.pipeline.FlattenedParseTree\"E\n\x16\x45valuateParserResponse\x12\n\n\x02\x66\x31\x18\x01 \x02(\x01\x12\x0f\n\x07kbestF1\x18\x02 \x01(\x01\x12\x0e\n\x06treeF1\x18\x03 \x03(\x01\"\xc8\x01\n\x0fTsurgeonRequest\x12H\n\noperations\x18\x01 \x03(\x0b\x32\x34.edu.stanford.nlp.pipeline.TsurgeonRequest.Operation\x12<\n\x05trees\x18\x02 \x03(\x0b\x32-.edu.stanford.nlp.pipeline.FlattenedParseTree\x1a-\n\tOperation\x12\x0e\n\x06tregex\x18\x01 \x02(\t\x12\x10\n\x08tsurgeon\x18\x02 \x03(\t\"P\n\x10TsurgeonResponse\x12<\n\x05trees\x18\x01 \x03(\x0b\x32-.edu.stanford.nlp.pipeline.FlattenedParseTree\"\x85\x01\n\x11MorphologyRequest\x12\x46\n\x05words\x18\x01 \x03(\x0b\x32\x37.edu.stanford.nlp.pipeline.MorphologyRequest.TaggedWord\x1a(\n\nTaggedWord\x12\x0c\n\x04word\x18\x01 \x02(\t\x12\x0c\n\x04xpos\x18\x02 \x01(\t\"\x9a\x01\n\x12MorphologyResponse\x12I\n\x05words\x18\x01 \x03(\x0b\x32:.edu.stanford.nlp.pipeline.MorphologyResponse.WordTagLemma\x1a\x39\n\x0cWordTagLemma\x12\x0c\n\x04word\x18\x01 \x02(\t\x12\x0c\n\x04xpos\x18\x02 \x01(\t\x12\r\n\x05lemma\x18\x03 \x02(\t\"Z\n\x1a\x44\x65pendencyConverterRequest\x12<\n\x05trees\x18\x01 \x03(\x0b\x32-.edu.stanford.nlp.pipeline.FlattenedParseTree\"\x90\x02\n\x1b\x44\x65pendencyConverterResponse\x12`\n\x0b\x63onversions\x18\x01 \x03(\x0b\x32K.edu.stanford.nlp.pipeline.DependencyConverterResponse.DependencyConversion\x1a\x8e\x01\n\x14\x44\x65pendencyConversion\x12\x39\n\x05graph\x18\x01 \x02(\x0b\x32*.edu.stanford.nlp.pipeline.DependencyGraph\x12;\n\x04tree\x18\x02 \x01(\x0b\x32-.edu.stanford.nlp.pipeline.FlattenedParseTree*\xa3\x01\n\x08Language\x12\x0b\n\x07Unknown\x10\x00\x12\x07\n\x03\x41ny\x10\x01\x12\n\n\x06\x41rabic\x10\x02\x12\x0b\n\x07\x43hinese\x10\x03\x12\x0b\n\x07\x45nglish\x10\x04\x12\n\n\x06German\x10\x05\x12\n\n\x06\x46rench\x10\x06\x12\n\n\x06Hebrew\x10\x07\x12\x0b\n\x07Spanish\x10\x08\x12\x14\n\x10UniversalEnglish\x10\t\x12\x14\n\x10UniversalChinese\x10\n*h\n\tSentiment\x12\x13\n\x0fSTRONG_NEGATIVE\x10\x00\x12\x11\n\rWEAK_NEGATIVE\x10\x01\x12\x0b\n\x07NEUTRAL\x10\x02\x12\x11\n\rWEAK_POSITIVE\x10\x03\x12\x13\n\x0fSTRONG_POSITIVE\x10\x04*\x93\x01\n\x14NaturalLogicRelation\x12\x0f\n\x0b\x45QUIVALENCE\x10\x00\x12\x16\n\x12\x46ORWARD_ENTAILMENT\x10\x01\x12\x16\n\x12REVERSE_ENTAILMENT\x10\x02\x12\x0c\n\x08NEGATION\x10\x03\x12\x0f\n\x0b\x41LTERNATION\x10\x04\x12\t\n\x05\x43OVER\x10\x05\x12\x10\n\x0cINDEPENDENCE\x10\x06\x42*\n\x19\x65\x64u.stanford.nlp.pipelineB\rCoreNLPProtos')

_LANGUAGE = DESCRIPTOR.enum_types_by_name['Language']
Language = enum_type_wrapper.EnumTypeWrapper(_LANGUAGE)
_SENTIMENT = DESCRIPTOR.enum_types_by_name['Sentiment']
Sentiment = enum_type_wrapper.EnumTypeWrapper(_SENTIMENT)
_NATURALLOGICRELATION = DESCRIPTOR.enum_types_by_name['NaturalLogicRelation']
NaturalLogicRelation = enum_type_wrapper.EnumTypeWrapper(_NATURALLOGICRELATION)
Unknown = 0
Any = 1
Arabic = 2
Chinese = 3
English = 4
German = 5
French = 6
Hebrew = 7
Spanish = 8
UniversalEnglish = 9
UniversalChinese = 10
STRONG_NEGATIVE = 0
WEAK_NEGATIVE = 1
NEUTRAL = 2
WEAK_POSITIVE = 3
STRONG_POSITIVE = 4
EQUIVALENCE = 0
FORWARD_ENTAILMENT = 1
REVERSE_ENTAILMENT = 2
NEGATION = 3
ALTERNATION = 4
COVER = 5
INDEPENDENCE = 6


_DOCUMENT = DESCRIPTOR.message_types_by_name['Document']
_SENTENCE = DESCRIPTOR.message_types_by_name['Sentence']
_TOKEN = DESCRIPTOR.message_types_by_name['Token']
_QUOTE = DESCRIPTOR.message_types_by_name['Quote']
_PARSETREE = DESCRIPTOR.message_types_by_name['ParseTree']
_DEPENDENCYGRAPH = DESCRIPTOR.message_types_by_name['DependencyGraph']
_DEPENDENCYGRAPH_NODE = _DEPENDENCYGRAPH.nested_types_by_name['Node']
_DEPENDENCYGRAPH_EDGE = _DEPENDENCYGRAPH.nested_types_by_name['Edge']
_COREFCHAIN = DESCRIPTOR.message_types_by_name['CorefChain']
_COREFCHAIN_COREFMENTION = _COREFCHAIN.nested_types_by_name['CorefMention']
_MENTION = DESCRIPTOR.message_types_by_name['Mention']
_INDEXEDWORD = DESCRIPTOR.message_types_by_name['IndexedWord']
_SPEAKERINFO = DESCRIPTOR.message_types_by_name['SpeakerInfo']
_SPAN = DESCRIPTOR.message_types_by_name['Span']
_TIMEX = DESCRIPTOR.message_types_by_name['Timex']
_ENTITY = DESCRIPTOR.message_types_by_name['Entity']
_RELATION = DESCRIPTOR.message_types_by_name['Relation']
_OPERATOR = DESCRIPTOR.message_types_by_name['Operator']
_POLARITY = DESCRIPTOR.message_types_by_name['Polarity']
_NERMENTION = DESCRIPTOR.message_types_by_name['NERMention']
_SENTENCEFRAGMENT = DESCRIPTOR.message_types_by_name['SentenceFragment']
_TOKENLOCATION = DESCRIPTOR.message_types_by_name['TokenLocation']
_RELATIONTRIPLE = DESCRIPTOR.message_types_by_name['RelationTriple']
_MAPSTRINGSTRING = DESCRIPTOR.message_types_by_name['MapStringString']
_MAPINTSTRING = DESCRIPTOR.message_types_by_name['MapIntString']
_SECTION = DESCRIPTOR.message_types_by_name['Section']
_SEMGREXREQUEST = DESCRIPTOR.message_types_by_name['SemgrexRequest']
_SEMGREXREQUEST_DEPENDENCIES = _SEMGREXREQUEST.nested_types_by_name['Dependencies']
_SEMGREXRESPONSE = DESCRIPTOR.message_types_by_name['SemgrexResponse']
_SEMGREXRESPONSE_NAMEDNODE = _SEMGREXRESPONSE.nested_types_by_name['NamedNode']
_SEMGREXRESPONSE_NAMEDRELATION = _SEMGREXRESPONSE.nested_types_by_name['NamedRelation']
_SEMGREXRESPONSE_NAMEDEDGE = _SEMGREXRESPONSE.nested_types_by_name['NamedEdge']
_SEMGREXRESPONSE_MATCH = _SEMGREXRESPONSE.nested_types_by_name['Match']
_SEMGREXRESPONSE_SEMGREXRESULT = _SEMGREXRESPONSE.nested_types_by_name['SemgrexResult']
_SEMGREXRESPONSE_GRAPHRESULT = _SEMGREXRESPONSE.nested_types_by_name['GraphResult']
_SSURGEONREQUEST = DESCRIPTOR.message_types_by_name['SsurgeonRequest']
_SSURGEONREQUEST_SSURGEON = _SSURGEONREQUEST.nested_types_by_name['Ssurgeon']
_SSURGEONRESPONSE = DESCRIPTOR.message_types_by_name['SsurgeonResponse']
_SSURGEONRESPONSE_SSURGEONRESULT = _SSURGEONRESPONSE.nested_types_by_name['SsurgeonResult']
_TOKENSREGEXREQUEST = DESCRIPTOR.message_types_by_name['TokensRegexRequest']
_TOKENSREGEXRESPONSE = DESCRIPTOR.message_types_by_name['TokensRegexResponse']
_TOKENSREGEXRESPONSE_MATCHLOCATION = _TOKENSREGEXRESPONSE.nested_types_by_name['MatchLocation']
_TOKENSREGEXRESPONSE_MATCH = _TOKENSREGEXRESPONSE.nested_types_by_name['Match']
_TOKENSREGEXRESPONSE_PATTERNMATCH = _TOKENSREGEXRESPONSE.nested_types_by_name['PatternMatch']
_DEPENDENCYENHANCERREQUEST = DESCRIPTOR.message_types_by_name['DependencyEnhancerRequest']
_FLATTENEDPARSETREE = DESCRIPTOR.message_types_by_name['FlattenedParseTree']
_FLATTENEDPARSETREE_NODE = _FLATTENEDPARSETREE.nested_types_by_name['Node']
_EVALUATEPARSERREQUEST = DESCRIPTOR.message_types_by_name['EvaluateParserRequest']
_EVALUATEPARSERREQUEST_PARSERESULT = _EVALUATEPARSERREQUEST.nested_types_by_name['ParseResult']
_EVALUATEPARSERRESPONSE = DESCRIPTOR.message_types_by_name['EvaluateParserResponse']
_TSURGEONREQUEST = DESCRIPTOR.message_types_by_name['TsurgeonRequest']
_TSURGEONREQUEST_OPERATION = _TSURGEONREQUEST.nested_types_by_name['Operation']
_TSURGEONRESPONSE = DESCRIPTOR.message_types_by_name['TsurgeonResponse']
_MORPHOLOGYREQUEST = DESCRIPTOR.message_types_by_name['MorphologyRequest']
_MORPHOLOGYREQUEST_TAGGEDWORD = _MORPHOLOGYREQUEST.nested_types_by_name['TaggedWord']
_MORPHOLOGYRESPONSE = DESCRIPTOR.message_types_by_name['MorphologyResponse']
_MORPHOLOGYRESPONSE_WORDTAGLEMMA = _MORPHOLOGYRESPONSE.nested_types_by_name['WordTagLemma']
_DEPENDENCYCONVERTERREQUEST = DESCRIPTOR.message_types_by_name['DependencyConverterRequest']
_DEPENDENCYCONVERTERRESPONSE = DESCRIPTOR.message_types_by_name['DependencyConverterResponse']
_DEPENDENCYCONVERTERRESPONSE_DEPENDENCYCONVERSION = _DEPENDENCYCONVERTERRESPONSE.nested_types_by_name['DependencyConversion']
Document = _reflection.GeneratedProtocolMessageType('Document', (_message.Message,), {
  'DESCRIPTOR' : _DOCUMENT,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.Document)
  })
_sym_db.RegisterMessage(Document)

Sentence = _reflection.GeneratedProtocolMessageType('Sentence', (_message.Message,), {
  'DESCRIPTOR' : _SENTENCE,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.Sentence)
  })
_sym_db.RegisterMessage(Sentence)

Token = _reflection.GeneratedProtocolMessageType('Token', (_message.Message,), {
  'DESCRIPTOR' : _TOKEN,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.Token)
  })
_sym_db.RegisterMessage(Token)

Quote = _reflection.GeneratedProtocolMessageType('Quote', (_message.Message,), {
  'DESCRIPTOR' : _QUOTE,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.Quote)
  })
_sym_db.RegisterMessage(Quote)

ParseTree = _reflection.GeneratedProtocolMessageType('ParseTree', (_message.Message,), {
  'DESCRIPTOR' : _PARSETREE,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.ParseTree)
  })
_sym_db.RegisterMessage(ParseTree)

DependencyGraph = _reflection.GeneratedProtocolMessageType('DependencyGraph', (_message.Message,), {

  'Node' : _reflection.GeneratedProtocolMessageType('Node', (_message.Message,), {
    'DESCRIPTOR' : _DEPENDENCYGRAPH_NODE,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.DependencyGraph.Node)
    })
  ,

  'Edge' : _reflection.GeneratedProtocolMessageType('Edge', (_message.Message,), {
    'DESCRIPTOR' : _DEPENDENCYGRAPH_EDGE,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.DependencyGraph.Edge)
    })
  ,
  'DESCRIPTOR' : _DEPENDENCYGRAPH,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.DependencyGraph)
  })
_sym_db.RegisterMessage(DependencyGraph)
_sym_db.RegisterMessage(DependencyGraph.Node)
_sym_db.RegisterMessage(DependencyGraph.Edge)

CorefChain = _reflection.GeneratedProtocolMessageType('CorefChain', (_message.Message,), {

  'CorefMention' : _reflection.GeneratedProtocolMessageType('CorefMention', (_message.Message,), {
    'DESCRIPTOR' : _COREFCHAIN_COREFMENTION,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.CorefChain.CorefMention)
    })
  ,
  'DESCRIPTOR' : _COREFCHAIN,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.CorefChain)
  })
_sym_db.RegisterMessage(CorefChain)
_sym_db.RegisterMessage(CorefChain.CorefMention)

Mention = _reflection.GeneratedProtocolMessageType('Mention', (_message.Message,), {
  'DESCRIPTOR' : _MENTION,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.Mention)
  })
_sym_db.RegisterMessage(Mention)

IndexedWord = _reflection.GeneratedProtocolMessageType('IndexedWord', (_message.Message,), {
  'DESCRIPTOR' : _INDEXEDWORD,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.IndexedWord)
  })
_sym_db.RegisterMessage(IndexedWord)

SpeakerInfo = _reflection.GeneratedProtocolMessageType('SpeakerInfo', (_message.Message,), {
  'DESCRIPTOR' : _SPEAKERINFO,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.SpeakerInfo)
  })
_sym_db.RegisterMessage(SpeakerInfo)

Span = _reflection.GeneratedProtocolMessageType('Span', (_message.Message,), {
  'DESCRIPTOR' : _SPAN,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.Span)
  })
_sym_db.RegisterMessage(Span)

Timex = _reflection.GeneratedProtocolMessageType('Timex', (_message.Message,), {
  'DESCRIPTOR' : _TIMEX,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.Timex)
  })
_sym_db.RegisterMessage(Timex)

Entity = _reflection.GeneratedProtocolMessageType('Entity', (_message.Message,), {
  'DESCRIPTOR' : _ENTITY,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.Entity)
  })
_sym_db.RegisterMessage(Entity)

Relation = _reflection.GeneratedProtocolMessageType('Relation', (_message.Message,), {
  'DESCRIPTOR' : _RELATION,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.Relation)
  })
_sym_db.RegisterMessage(Relation)

Operator = _reflection.GeneratedProtocolMessageType('Operator', (_message.Message,), {
  'DESCRIPTOR' : _OPERATOR,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.Operator)
  })
_sym_db.RegisterMessage(Operator)

Polarity = _reflection.GeneratedProtocolMessageType('Polarity', (_message.Message,), {
  'DESCRIPTOR' : _POLARITY,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.Polarity)
  })
_sym_db.RegisterMessage(Polarity)

NERMention = _reflection.GeneratedProtocolMessageType('NERMention', (_message.Message,), {
  'DESCRIPTOR' : _NERMENTION,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.NERMention)
  })
_sym_db.RegisterMessage(NERMention)

SentenceFragment = _reflection.GeneratedProtocolMessageType('SentenceFragment', (_message.Message,), {
  'DESCRIPTOR' : _SENTENCEFRAGMENT,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.SentenceFragment)
  })
_sym_db.RegisterMessage(SentenceFragment)

TokenLocation = _reflection.GeneratedProtocolMessageType('TokenLocation', (_message.Message,), {
  'DESCRIPTOR' : _TOKENLOCATION,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.TokenLocation)
  })
_sym_db.RegisterMessage(TokenLocation)

RelationTriple = _reflection.GeneratedProtocolMessageType('RelationTriple', (_message.Message,), {
  'DESCRIPTOR' : _RELATIONTRIPLE,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.RelationTriple)
  })
_sym_db.RegisterMessage(RelationTriple)

MapStringString = _reflection.GeneratedProtocolMessageType('MapStringString', (_message.Message,), {
  'DESCRIPTOR' : _MAPSTRINGSTRING,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.MapStringString)
  })
_sym_db.RegisterMessage(MapStringString)

MapIntString = _reflection.GeneratedProtocolMessageType('MapIntString', (_message.Message,), {
  'DESCRIPTOR' : _MAPINTSTRING,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.MapIntString)
  })
_sym_db.RegisterMessage(MapIntString)

Section = _reflection.GeneratedProtocolMessageType('Section', (_message.Message,), {
  'DESCRIPTOR' : _SECTION,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.Section)
  })
_sym_db.RegisterMessage(Section)

SemgrexRequest = _reflection.GeneratedProtocolMessageType('SemgrexRequest', (_message.Message,), {

  'Dependencies' : _reflection.GeneratedProtocolMessageType('Dependencies', (_message.Message,), {
    'DESCRIPTOR' : _SEMGREXREQUEST_DEPENDENCIES,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.SemgrexRequest.Dependencies)
    })
  ,
  'DESCRIPTOR' : _SEMGREXREQUEST,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.SemgrexRequest)
  })
_sym_db.RegisterMessage(SemgrexRequest)
_sym_db.RegisterMessage(SemgrexRequest.Dependencies)

SemgrexResponse = _reflection.GeneratedProtocolMessageType('SemgrexResponse', (_message.Message,), {

  'NamedNode' : _reflection.GeneratedProtocolMessageType('NamedNode', (_message.Message,), {
    'DESCRIPTOR' : _SEMGREXRESPONSE_NAMEDNODE,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.SemgrexResponse.NamedNode)
    })
  ,

  'NamedRelation' : _reflection.GeneratedProtocolMessageType('NamedRelation', (_message.Message,), {
    'DESCRIPTOR' : _SEMGREXRESPONSE_NAMEDRELATION,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.SemgrexResponse.NamedRelation)
    })
  ,

  'NamedEdge' : _reflection.GeneratedProtocolMessageType('NamedEdge', (_message.Message,), {
    'DESCRIPTOR' : _SEMGREXRESPONSE_NAMEDEDGE,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.SemgrexResponse.NamedEdge)
    })
  ,

  'Match' : _reflection.GeneratedProtocolMessageType('Match', (_message.Message,), {
    'DESCRIPTOR' : _SEMGREXRESPONSE_MATCH,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.SemgrexResponse.Match)
    })
  ,

  'SemgrexResult' : _reflection.GeneratedProtocolMessageType('SemgrexResult', (_message.Message,), {
    'DESCRIPTOR' : _SEMGREXRESPONSE_SEMGREXRESULT,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.SemgrexResponse.SemgrexResult)
    })
  ,

  'GraphResult' : _reflection.GeneratedProtocolMessageType('GraphResult', (_message.Message,), {
    'DESCRIPTOR' : _SEMGREXRESPONSE_GRAPHRESULT,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.SemgrexResponse.GraphResult)
    })
  ,
  'DESCRIPTOR' : _SEMGREXRESPONSE,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.SemgrexResponse)
  })
_sym_db.RegisterMessage(SemgrexResponse)
_sym_db.RegisterMessage(SemgrexResponse.NamedNode)
_sym_db.RegisterMessage(SemgrexResponse.NamedRelation)
_sym_db.RegisterMessage(SemgrexResponse.NamedEdge)
_sym_db.RegisterMessage(SemgrexResponse.Match)
_sym_db.RegisterMessage(SemgrexResponse.SemgrexResult)
_sym_db.RegisterMessage(SemgrexResponse.GraphResult)

SsurgeonRequest = _reflection.GeneratedProtocolMessageType('SsurgeonRequest', (_message.Message,), {

  'Ssurgeon' : _reflection.GeneratedProtocolMessageType('Ssurgeon', (_message.Message,), {
    'DESCRIPTOR' : _SSURGEONREQUEST_SSURGEON,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.SsurgeonRequest.Ssurgeon)
    })
  ,
  'DESCRIPTOR' : _SSURGEONREQUEST,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.SsurgeonRequest)
  })
_sym_db.RegisterMessage(SsurgeonRequest)
_sym_db.RegisterMessage(SsurgeonRequest.Ssurgeon)

SsurgeonResponse = _reflection.GeneratedProtocolMessageType('SsurgeonResponse', (_message.Message,), {

  'SsurgeonResult' : _reflection.GeneratedProtocolMessageType('SsurgeonResult', (_message.Message,), {
    'DESCRIPTOR' : _SSURGEONRESPONSE_SSURGEONRESULT,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.SsurgeonResponse.SsurgeonResult)
    })
  ,
  'DESCRIPTOR' : _SSURGEONRESPONSE,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.SsurgeonResponse)
  })
_sym_db.RegisterMessage(SsurgeonResponse)
_sym_db.RegisterMessage(SsurgeonResponse.SsurgeonResult)

TokensRegexRequest = _reflection.GeneratedProtocolMessageType('TokensRegexRequest', (_message.Message,), {
  'DESCRIPTOR' : _TOKENSREGEXREQUEST,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.TokensRegexRequest)
  })
_sym_db.RegisterMessage(TokensRegexRequest)

TokensRegexResponse = _reflection.GeneratedProtocolMessageType('TokensRegexResponse', (_message.Message,), {

  'MatchLocation' : _reflection.GeneratedProtocolMessageType('MatchLocation', (_message.Message,), {
    'DESCRIPTOR' : _TOKENSREGEXRESPONSE_MATCHLOCATION,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.TokensRegexResponse.MatchLocation)
    })
  ,

  'Match' : _reflection.GeneratedProtocolMessageType('Match', (_message.Message,), {
    'DESCRIPTOR' : _TOKENSREGEXRESPONSE_MATCH,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.TokensRegexResponse.Match)
    })
  ,

  'PatternMatch' : _reflection.GeneratedProtocolMessageType('PatternMatch', (_message.Message,), {
    'DESCRIPTOR' : _TOKENSREGEXRESPONSE_PATTERNMATCH,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.TokensRegexResponse.PatternMatch)
    })
  ,
  'DESCRIPTOR' : _TOKENSREGEXRESPONSE,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.TokensRegexResponse)
  })
_sym_db.RegisterMessage(TokensRegexResponse)
_sym_db.RegisterMessage(TokensRegexResponse.MatchLocation)
_sym_db.RegisterMessage(TokensRegexResponse.Match)
_sym_db.RegisterMessage(TokensRegexResponse.PatternMatch)

DependencyEnhancerRequest = _reflection.GeneratedProtocolMessageType('DependencyEnhancerRequest', (_message.Message,), {
  'DESCRIPTOR' : _DEPENDENCYENHANCERREQUEST,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.DependencyEnhancerRequest)
  })
_sym_db.RegisterMessage(DependencyEnhancerRequest)

FlattenedParseTree = _reflection.GeneratedProtocolMessageType('FlattenedParseTree', (_message.Message,), {

  'Node' : _reflection.GeneratedProtocolMessageType('Node', (_message.Message,), {
    'DESCRIPTOR' : _FLATTENEDPARSETREE_NODE,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.FlattenedParseTree.Node)
    })
  ,
  'DESCRIPTOR' : _FLATTENEDPARSETREE,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.FlattenedParseTree)
  })
_sym_db.RegisterMessage(FlattenedParseTree)
_sym_db.RegisterMessage(FlattenedParseTree.Node)

EvaluateParserRequest = _reflection.GeneratedProtocolMessageType('EvaluateParserRequest', (_message.Message,), {

  'ParseResult' : _reflection.GeneratedProtocolMessageType('ParseResult', (_message.Message,), {
    'DESCRIPTOR' : _EVALUATEPARSERREQUEST_PARSERESULT,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.EvaluateParserRequest.ParseResult)
    })
  ,
  'DESCRIPTOR' : _EVALUATEPARSERREQUEST,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.EvaluateParserRequest)
  })
_sym_db.RegisterMessage(EvaluateParserRequest)
_sym_db.RegisterMessage(EvaluateParserRequest.ParseResult)

EvaluateParserResponse = _reflection.GeneratedProtocolMessageType('EvaluateParserResponse', (_message.Message,), {
  'DESCRIPTOR' : _EVALUATEPARSERRESPONSE,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.EvaluateParserResponse)
  })
_sym_db.RegisterMessage(EvaluateParserResponse)

TsurgeonRequest = _reflection.GeneratedProtocolMessageType('TsurgeonRequest', (_message.Message,), {

  'Operation' : _reflection.GeneratedProtocolMessageType('Operation', (_message.Message,), {
    'DESCRIPTOR' : _TSURGEONREQUEST_OPERATION,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.TsurgeonRequest.Operation)
    })
  ,
  'DESCRIPTOR' : _TSURGEONREQUEST,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.TsurgeonRequest)
  })
_sym_db.RegisterMessage(TsurgeonRequest)
_sym_db.RegisterMessage(TsurgeonRequest.Operation)

TsurgeonResponse = _reflection.GeneratedProtocolMessageType('TsurgeonResponse', (_message.Message,), {
  'DESCRIPTOR' : _TSURGEONRESPONSE,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.TsurgeonResponse)
  })
_sym_db.RegisterMessage(TsurgeonResponse)

MorphologyRequest = _reflection.GeneratedProtocolMessageType('MorphologyRequest', (_message.Message,), {

  'TaggedWord' : _reflection.GeneratedProtocolMessageType('TaggedWord', (_message.Message,), {
    'DESCRIPTOR' : _MORPHOLOGYREQUEST_TAGGEDWORD,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.MorphologyRequest.TaggedWord)
    })
  ,
  'DESCRIPTOR' : _MORPHOLOGYREQUEST,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.MorphologyRequest)
  })
_sym_db.RegisterMessage(MorphologyRequest)
_sym_db.RegisterMessage(MorphologyRequest.TaggedWord)

MorphologyResponse = _reflection.GeneratedProtocolMessageType('MorphologyResponse', (_message.Message,), {

  'WordTagLemma' : _reflection.GeneratedProtocolMessageType('WordTagLemma', (_message.Message,), {
    'DESCRIPTOR' : _MORPHOLOGYRESPONSE_WORDTAGLEMMA,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.MorphologyResponse.WordTagLemma)
    })
  ,
  'DESCRIPTOR' : _MORPHOLOGYRESPONSE,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.MorphologyResponse)
  })
_sym_db.RegisterMessage(MorphologyResponse)
_sym_db.RegisterMessage(MorphologyResponse.WordTagLemma)

DependencyConverterRequest = _reflection.GeneratedProtocolMessageType('DependencyConverterRequest', (_message.Message,), {
  'DESCRIPTOR' : _DEPENDENCYCONVERTERREQUEST,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.DependencyConverterRequest)
  })
_sym_db.RegisterMessage(DependencyConverterRequest)

DependencyConverterResponse = _reflection.GeneratedProtocolMessageType('DependencyConverterResponse', (_message.Message,), {

  'DependencyConversion' : _reflection.GeneratedProtocolMessageType('DependencyConversion', (_message.Message,), {
    'DESCRIPTOR' : _DEPENDENCYCONVERTERRESPONSE_DEPENDENCYCONVERSION,
    '__module__' : 'CoreNLP_pb2'
    # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.DependencyConverterResponse.DependencyConversion)
    })
  ,
  'DESCRIPTOR' : _DEPENDENCYCONVERTERRESPONSE,
  '__module__' : 'CoreNLP_pb2'
  # @@protoc_insertion_point(class_scope:edu.stanford.nlp.pipeline.DependencyConverterResponse)
  })
_sym_db.RegisterMessage(DependencyConverterResponse)
_sym_db.RegisterMessage(DependencyConverterResponse.DependencyConversion)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031edu.stanford.nlp.pipelineB\rCoreNLPProtos'
  _DEPENDENCYGRAPH.fields_by_name['root']._options = None
  _DEPENDENCYGRAPH.fields_by_name['root']._serialized_options = b'\020\001'
  _DEPENDENCYGRAPH.fields_by_name['rootNode']._options = None
  _DEPENDENCYGRAPH.fields_by_name['rootNode']._serialized_options = b'\020\001'
  _LANGUAGE._serialized_start=13457
  _LANGUAGE._serialized_end=13620
  _SENTIMENT._serialized_start=13622
  _SENTIMENT._serialized_end=13726
  _NATURALLOGICRELATION._serialized_start=13729
  _NATURALLOGICRELATION._serialized_end=13876
  _DOCUMENT._serialized_start=45
  _DOCUMENT._serialized_end=782
  _SENTENCE._serialized_start=785
  _SENTENCE._serialized_end=2820
  _TOKEN._serialized_start=2823
  _TOKEN._serialized_end=4477
  _QUOTE._serialized_start=4480
  _QUOTE._serialized_end=4964
  _PARSETREE._serialized_start=4967
  _PARSETREE._serialized_end=5166
  _DEPENDENCYGRAPH._serialized_start=5169
  _DEPENDENCYGRAPH._serialized_end=5708
  _DEPENDENCYGRAPH_NODE._serialized_start=5403
  _DEPENDENCYGRAPH_NODE._serialized_end=5491
  _DEPENDENCYGRAPH_EDGE._serialized_start=5494
  _DEPENDENCYGRAPH_EDGE._serialized_end=5708
  _COREFCHAIN._serialized_start=5711
  _COREFCHAIN._serialized_end=6037
  _COREFCHAIN_COREFMENTION._serialized_start=5836
  _COREFCHAIN_COREFMENTION._serialized_end=6037
  _MENTION._serialized_start=6040
  _MENTION._serialized_end=7175
  _INDEXEDWORD._serialized_start=7177
  _INDEXEDWORD._serialized_end=7265
  _SPEAKERINFO._serialized_start=7267
  _SPEAKERINFO._serialized_end=7319
  _SPAN._serialized_start=7321
  _SPAN._serialized_end=7355
  _TIMEX._serialized_start=7357
  _TIMEX._serialized_end=7476
  _ENTITY._serialized_start=7479
  _ENTITY._serialized_end=7698
  _RELATION._serialized_start=7701
  _RELATION._serialized_end=7884
  _OPERATOR._serialized_start=7887
  _OPERATOR._serialized_end=8065
  _POLARITY._serialized_start=8068
  _POLARITY._serialized_end=8621
  _NERMENTION._serialized_start=8624
  _NERMENTION._serialized_end=8973
  _SENTENCEFRAGMENT._serialized_start=8975
  _SENTENCEFRAGMENT._serialized_end=9064
  _TOKENLOCATION._serialized_start=9066
  _TOKENLOCATION._serialized_end=9124
  _RELATIONTRIPLE._serialized_start=9127
  _RELATIONTRIPLE._serialized_end=9537
  _MAPSTRINGSTRING._serialized_start=9539
  _MAPSTRINGSTRING._serialized_end=9584
  _MAPINTSTRING._serialized_start=9586
  _MAPINTSTRING._serialized_end=9628
  _SECTION._serialized_start=9631
  _SECTION._serialized_end=9883
  _SEMGREXREQUEST._serialized_start=9886
  _SEMGREXREQUEST._serialized_end=10114
  _SEMGREXREQUEST_DEPENDENCIES._serialized_start=9992
  _SEMGREXREQUEST_DEPENDENCIES._serialized_end=10114
  _SEMGREXRESPONSE._serialized_start=10117
  _SEMGREXRESPONSE._serialized_end=10880
  _SEMGREXRESPONSE_NAMEDNODE._serialized_start=10208
  _SEMGREXRESPONSE_NAMEDNODE._serialized_end=10253
  _SEMGREXRESPONSE_NAMEDRELATION._serialized_start=10255
  _SEMGREXRESPONSE_NAMEDRELATION._serialized_end=10298
  _SEMGREXRESPONSE_NAMEDEDGE._serialized_start=10301
  _SEMGREXRESPONSE_NAMEDEDGE._serialized_end=10429
  _SEMGREXRESPONSE_MATCH._serialized_start=10432
  _SEMGREXRESPONSE_MATCH._serialized_end=10709
  _SEMGREXRESPONSE_SEMGREXRESULT._serialized_start=10711
  _SEMGREXRESPONSE_SEMGREXRESULT._serialized_end=10791
  _SEMGREXRESPONSE_GRAPHRESULT._serialized_start=10793
  _SEMGREXRESPONSE_GRAPHRESULT._serialized_end=10880
  _SSURGEONREQUEST._serialized_start=10883
  _SSURGEONREQUEST._serialized_end=11123
  _SSURGEONREQUEST_SSURGEON._serialized_start=11032
  _SSURGEONREQUEST_SSURGEON._serialized_end=11123
  _SSURGEONRESPONSE._serialized_start=11126
  _SSURGEONRESPONSE._serialized_end=11314
  _SSURGEONRESPONSE_SSURGEONRESULT._serialized_start=11222
  _SSURGEONRESPONSE_SSURGEONRESULT._serialized_end=11314
  _TOKENSREGEXREQUEST._serialized_start=11316
  _TOKENSREGEXREQUEST._serialized_end=11403
  _TOKENSREGEXRESPONSE._serialized_start=11406
  _TOKENSREGEXRESPONSE._serialized_end=11829
  _TOKENSREGEXRESPONSE_MATCHLOCATION._serialized_start=11505
  _TOKENSREGEXRESPONSE_MATCHLOCATION._serialized_end=11562
  _TOKENSREGEXRESPONSE_MATCH._serialized_start=11565
  _TOKENSREGEXRESPONSE_MATCH._serialized_end=11744
  _TOKENSREGEXRESPONSE_PATTERNMATCH._serialized_start=11746
  _TOKENSREGEXRESPONSE_PATTERNMATCH._serialized_end=11829
  _DEPENDENCYENHANCERREQUEST._serialized_start=11832
  _DEPENDENCYENHANCERREQUEST._serialized_end=12006
  _FLATTENEDPARSETREE._serialized_start=12009
  _FLATTENEDPARSETREE._serialized_end=12189
  _FLATTENEDPARSETREE_NODE._serialized_start=12098
  _FLATTENEDPARSETREE_NODE._serialized_end=12189
  _EVALUATEPARSERREQUEST._serialized_start=12192
  _EVALUATEPARSERREQUEST._serialized_end=12438
  _EVALUATEPARSERREQUEST_PARSERESULT._serialized_start=12298
  _EVALUATEPARSERREQUEST_PARSERESULT._serialized_end=12438
  _EVALUATEPARSERRESPONSE._serialized_start=12440
  _EVALUATEPARSERRESPONSE._serialized_end=12509
  _TSURGEONREQUEST._serialized_start=12512
  _TSURGEONREQUEST._serialized_end=12712
  _TSURGEONREQUEST_OPERATION._serialized_start=12667
  _TSURGEONREQUEST_OPERATION._serialized_end=12712
  _TSURGEONRESPONSE._serialized_start=12714
  _TSURGEONRESPONSE._serialized_end=12794
  _MORPHOLOGYREQUEST._serialized_start=12797
  _MORPHOLOGYREQUEST._serialized_end=12930
  _MORPHOLOGYREQUEST_TAGGEDWORD._serialized_start=12890
  _MORPHOLOGYREQUEST_TAGGEDWORD._serialized_end=12930
  _MORPHOLOGYRESPONSE._serialized_start=12933
  _MORPHOLOGYRESPONSE._serialized_end=13087
  _MORPHOLOGYRESPONSE_WORDTAGLEMMA._serialized_start=13030
  _MORPHOLOGYRESPONSE_WORDTAGLEMMA._serialized_end=13087
  _DEPENDENCYCONVERTERREQUEST._serialized_start=13089
  _DEPENDENCYCONVERTERREQUEST._serialized_end=13179
  _DEPENDENCYCONVERTERRESPONSE._serialized_start=13182
  _DEPENDENCYCONVERTERRESPONSE._serialized_end=13454
  _DEPENDENCYCONVERTERRESPONSE_DEPENDENCYCONVERSION._serialized_start=13312
  _DEPENDENCYCONVERTERRESPONSE_DEPENDENCYCONVERSION._serialized_end=13454
# @@protoc_insertion_point(module_scope)



import argparse
import json
from operator import itemgetter
import os

import stanza

from stanza.utils.default_paths import get_default_paths
from stanza.utils.get_tqdm import get_tqdm
from stanza.utils.datasets.coref.utils import process_document

tqdm = get_tqdm()

def flatten_spans(coref_spans):
    """
    Put span IDs on each span, then flatten them into a single list sorted by first word
    """
    # put span indices on the spans
    #   [[[38, 39], [42, 43], [41, 41], [180, 180], [300, 300]], [[60, 68],
    #   -->
    #   [[[0, 38, 39], [0, 42, 43], [0, 41, 41], [0, 180, 180], [0, 300, 300]], [[1, 60, 68], ...
    coref_spans = [[[span_idx, x, y] for x, y in span] for span_idx, span in enumerate(coref_spans)]
    # flatten list
    #   -->
    #   [[0, 38, 39], [0, 42, 43], [0, 41, 41], [0, 180, 180], [0, 300, 300], [1, 60, 68], ...
    coref_spans = [y for x in coref_spans for y in x]
    # sort by the first word index
    #   -->
    #   [[0, 38, 39], [0, 41, 41], [0, 42, 43], [1, 60, 68], [0, 180, 180], [0, 300, 300], ...
    coref_spans = sorted(coref_spans, key=itemgetter(1))
    return coref_spans

def remove_nulls(coref_spans, sentences):
    """
    Removes the "" and "NULL" words from the sentences

    Also, reindex the spans by the number of words removed.
    So, we might get something like
      [[0, 2], [31, 33], [134, 136], [161, 162]]
      ->
      [[0, 2], [30, 32], [129, 131], [155, 156]]
    """
    word_map = []
    word_idx = 0
    map_idx = 0
    new_sentences = []
    for sentence in sentences:
        new_sentence = []
        for word in sentence:
            word_map.append(map_idx)
            word_idx += 1
            if word != '' and word != 'NULL':
                new_sentence.append(word)
                map_idx += 1
        new_sentences.append(new_sentence)

    new_spans = []
    for mention in coref_spans:
        new_mention = []
        for span in mention:
            span = [word_map[x] for x in span]
            new_mention.append(span)
        new_spans.append(new_mention)
    return new_spans, new_sentences

def arrange_spans_by_sentence(coref_spans, sentences):
    sentence_spans = []

    current_index = 0
    span_idx = 0
    for sentence in sentences:
        current_sentence_spans = []
        end_index = current_index + len(sentence)
        while span_idx < len(coref_spans) and coref_spans[span_idx][1] < end_index:
            new_span = [coref_spans[span_idx][0], coref_spans[span_idx][1] - current_index, coref_spans[span_idx][2] - current_index]
            current_sentence_spans.append(new_span)
            span_idx += 1
        sentence_spans.append(current_sentence_spans)
        current_index = end_index
    return sentence_spans

def convert_dataset_section(pipe, section, use_cconj_heads):
    """
    Reprocess the original data into a format compatible with previous conversion utilities

    - remove blank and NULL words
    - rearrange the spans into spans per sentence instead of a list of indices for each span
    - process the document using a Hindi pipeline
    """
    processed_section = []

    for idx, doc in enumerate(tqdm(section)):
        doc_id = doc['doc_key']
        part_id = ""
        sentences = doc['sentences']
        sentence_speakers = doc['speakers']

        coref_spans = doc['clusters']
        coref_spans, sentences = remove_nulls(coref_spans, sentences)
        coref_spans = flatten_spans(coref_spans)
        coref_spans = arrange_spans_by_sentence(coref_spans, sentences)

        processed = process_document(pipe, doc_id, part_id, sentences, coref_spans, sentence_speakers, use_cconj_heads=use_cconj_heads)
        processed_section.append(processed)
    return processed_section

def remove_nulls_dataset_section(section):
    processed_section = []
    for doc in section:
        sentences = doc['sentences']
        coref_spans = doc['clusters']
        coref_spans, sentences = remove_nulls(coref_spans, sentences)
        doc['sentences'] = sentences
        doc['clusters'] = coref_spans
        processed_section.append(doc)
    return processed_section


def read_json_file(filename):
    with open(filename, encoding="utf-8") as fin:
        dataset = []
        for line in fin:
            line = line.strip()
            if not line:
                continue
            dataset.append(json.loads(line))
    return dataset

def write_json_file(output_filename, converted_section):
    with open(output_filename, "w", encoding="utf-8") as fout:
        json.dump(converted_section, fout, indent=2)

def main():
    parser = argparse.ArgumentParser(
        prog='Convert Hindi Coref Data',
    )
    parser.add_argument('--no_use_cconj_heads', dest='use_cconj_heads', action='store_false', help="Don't use the conjunction-aware transformation")
    parser.add_argument('--remove_nulls', action='store_true', help="The only action is to remove the NULLs and blank tokens")
    args = parser.parse_args()

    paths = get_default_paths()
    coref_input_path = paths["COREF_BASE"]
    hindi_base_path = os.path.join(coref_input_path, "hindi", "dataset")

    sections = ("train", "dev", "test")
    if args.remove_nulls:
        for section in sections:
            input_filename = os.path.join(hindi_base_path, "%s.hindi.jsonlines" % section)
            dataset = read_json_file(input_filename)
            dataset = remove_nulls_dataset_section(dataset)
            output_filename = os.path.join(hindi_base_path, "hi_deeph.%s.nonulls.json" % section)
            with open(output_filename, "w", encoding="utf-8") as fout:
                for doc in dataset:
                    json.dump(doc, fout, ensure_ascii=False)
                    fout.write("\n")
    else:
        pipe = stanza.Pipeline("hi", processors="tokenize,pos,lemma,depparse", package="default_accurate", tokenize_pretokenized=True)

        os.makedirs(paths["COREF_DATA_DIR"], exist_ok=True)

        for section in sections:
            input_filename = os.path.join(hindi_base_path, "%s.hindi.jsonlines" % section)
            dataset = read_json_file(input_filename)

            output_filename = os.path.join(paths["COREF_DATA_DIR"], "hi_deeph.%s.json" % section)
            converted_section = convert_dataset_section(pipe, dataset, use_cconj_heads=args.use_cconj_heads)
            write_json_file(output_filename, converted_section)

if __name__ == '__main__':
    main()



from collections import deque
import subprocess

from stanza.models.common.utils import misc_to_space_after
from stanza.models.constituency.parse_tree import Tree
from stanza.protobuf import DependencyGraph, FlattenedParseTree
from stanza.server.client import resolve_classpath

def send_request(request, response_type, java_main, classpath=None):
    """
    Use subprocess to run a Java protobuf processor on the given request

    Returns the protobuf response
    """
    classpath = resolve_classpath(classpath)
    if classpath is None:
        raise ValueError("Classpath is None,  Perhaps you need to set the $CLASSPATH or $CORENLP_HOME environment variable to point to a CoreNLP install.")
    pipe = subprocess.run(["java", "-cp", classpath, java_main],
                          input=request.SerializeToString(),
                          stdout=subprocess.PIPE,
                          check=True)
    response = response_type()
    response.ParseFromString(pipe.stdout)
    return response

def add_tree_nodes(proto_tree, tree, score):
    # add an open node
    node = proto_tree.nodes.add()
    node.openNode = True
    if score is not None:
        node.score = score

    # add the content of this node
    node = proto_tree.nodes.add()
    node.value = tree.label

    # add all children...
    # leaves get just one node
    # branches are called recursively
    for child in tree.children:
        if child.is_leaf():
            node = proto_tree.nodes.add()
            node.value = child.label
        else:
            add_tree_nodes(proto_tree, child, None)

    node = proto_tree.nodes.add()
    node.closeNode = True

def build_tree(tree, score):
    """
    Builds a FlattenedParseTree from CoreNLP.proto

    Populates the value field from tree.label and iterates through the
    children via tree.children.  Should work on any tree structure
    which follows that layout

    The score will be added to the top node (if it is not None)

    Operates by recursively calling add_tree_nodes
    """
    proto_tree = FlattenedParseTree()
    add_tree_nodes(proto_tree, tree, score)
    return proto_tree

def from_tree(proto_tree):
    """
    Convert a FlattenedParseTree back into a Tree

    returns Tree, score
      (score might be None if it is missing)
    """
    score = None
    stack = deque()
    for node in proto_tree.nodes:
        if node.HasField("score") and score is None:
            score = node.score

        if node.openNode:
            if len(stack) > 0 and isinstance(stack[-1], FlattenedParseTree.Node) and stack[-1].openNode:
                raise ValueError("Got a proto with no label on a node: {}".format(proto_tree))
            stack.append(node)
            continue
        if not node.closeNode:
            child = Tree(label=node.value)
            # TODO: do something with the score
            stack.append(child)
            continue

        # must be a close operation...
        if len(stack) <= 1:
            raise ValueError("Got a proto with too many close operations: {}".format(proto_tree))
        # on a close operation, pop until we hit the open
        # then turn everything in that span into a new node
        children = []
        nextNode = stack.pop()
        while not isinstance(nextNode, FlattenedParseTree.Node):
            children.append(nextNode)
            nextNode = stack.pop()
        if len(children) == 0:
            raise ValueError("Got a proto with an open immediately followed by a close: {}".format(proto_tree))
        children.reverse()
        label = children[0]
        children = children[1:]
        subtree = Tree(label=label.label, children=children)
        stack.append(subtree)

    if len(stack) > 1:
        raise ValueError("Got a proto which does not close all of the nodes: {}".format(proto_tree))
    tree = stack.pop()
    if not isinstance(tree, Tree):
        raise ValueError("Got a proto which was just one Open operation: {}".format(proto_tree))
    return tree, score

def add_token(token_list, word, token):
    """
    Add a token to a proto request.

    CoreNLP tokens have components of both word and token from stanza.

    We pass along "after" but not "before"
    """
    if token is None and isinstance(word.id, int):
        raise AssertionError("Only expected word w/o token for 'extra' words")

    query_token = token_list.add()
    query_token.word = word.text
    query_token.value = word.text
    if word.lemma is not None:
        query_token.lemma = word.lemma
    if word.xpos is not None:
        query_token.pos = word.xpos
    if word.upos is not None:
        query_token.coarseTag = word.upos
    if word.feats and word.feats != "_":
        for feature in word.feats.split("|"):
            key, value = feature.split("=", maxsplit=1)
            query_token.conllUFeatures.key.append(key)
            query_token.conllUFeatures.value.append(value)
    if token is not None:
        if token.ner is not None:
            query_token.ner = token.ner
        if token is not None and len(token.id) > 1:
            query_token.mwtText = token.text
            query_token.isMWT = True
            query_token.isFirstMWT = token.id[0] == word.id
        if token.id[-1] != word.id:
            # if we are not the last word of an MWT token
            # we are absolutely not followed by space
            pass
        else:
            query_token.after = token.spaces_after

        query_token.index = word.id
    else:
        # presumably empty words won't really be written this way,
        # but we can still keep track of it
        query_token.after = misc_to_space_after(word.misc)

        query_token.index = word.id[0]
        query_token.emptyIndex = word.id[1]

    if word.misc and word.misc != "_":
        query_token.conllUMisc = word.misc
    if token is not None and token.misc and token.misc != "_":
        query_token.mwtMisc = token.misc

def add_sentence(request_sentences, sentence, num_tokens):
    """
    Add the tokens for this stanza sentence to a list of protobuf sentences
    """
    request_sentence = request_sentences.add()
    request_sentence.tokenOffsetBegin = num_tokens
    request_sentence.tokenOffsetEnd = num_tokens + sum(len(token.words) for token in sentence.tokens)
    for token in sentence.tokens:
        for word in token.words:
            add_token(request_sentence.token, word, token)
    return request_sentence

def add_word_to_graph(graph, word, sent_idx):
    """
    Add a node and possibly an edge for a word in a basic dependency graph.
    """
    node = graph.node.add()
    node.sentenceIndex = sent_idx+1
    if isinstance(word.id, int):
        node.index = word.id
    else:
        node.index = word.id[0]
        node.emptyIndex = word.id[1]

    if word.head != 0 and word.head is not None:
        edge = graph.edge.add()
        edge.source = word.head
        if isinstance(word.id, int):
            edge.target = word.id
        else:
            edge.target = word.id[0]
            edge.targetEmpty = word.id[1]
        if word.deprel is not None:
            edge.dep = word.deprel
        else:
            # the receiving side doesn't like null as a dependency
            edge.dep = "_"

def convert_networkx_graph(graph_proto, sentence, sent_idx):
    """
    Turns a networkx graph into a DependencyGraph from the proto file
    """
    for token in sentence.tokens:
        for word in token.words:
            add_token(graph_proto.token, word, token)
    for word in sentence.empty_words:
        add_token(graph_proto.token, word, None)

    dependencies = sentence._enhanced_dependencies
    for target in dependencies:
        if target == 0:
            # don't need to send the explicit root
            continue
        for source in dependencies.predecessors(target):
            if source == 0:
                # unlike with basic, we need to send over the roots,
                # as the enhanced can have loops
                graph_proto.rootNode.append(len(graph_proto.node))
                continue
            for deprel in dependencies.get_edge_data(source, target):
                edge = graph_proto.edge.add()
                if isinstance(source, int):
                    edge.source = source
                else:
                    edge.source = source[0]
                    if source[1] != 0:
                        edge.sourceEmpty = source[1]
                if isinstance(target, int):
                    edge.target = target
                else:
                    edge.target = target[0]
                    if target[1] != 0:
                        edge.targetEmpty = target[1]
                edge.dep = deprel
        node = graph_proto.node.add()
        node.sentenceIndex = sent_idx + 1
        # the nodes in the networkx graph are indexed from 1, not counting the root
        if isinstance(target, int):
            node.index = target
        else:
            node.index = target[0]
            if target[1] != 0:
                node.emptyIndex = target[1]
    return graph_proto

def features_to_string(features):
    if not features:
        return None
    if len(features.key) == 0:
        return None
    return "|".join("%s=%s" % (key, value) for key, value in zip(features.key, features.value))

def misc_space_pieces(misc):
    """
    Return only the space-related misc pieces
    """
    if misc is None or misc == "" or misc == "_":
        return misc
    pieces = misc.split("|")
    pieces = [x for x in pieces if x.split("=", maxsplit=1)[0] in ("SpaceAfter", "SpacesAfter", "SpacesBefore")]
    if len(pieces) > 0:
        return "|".join(pieces)
    return None

def remove_space_misc(misc):
    """
    Remove any pieces from misc which are space-related
    """
    if misc is None or misc == "" or misc == "_":
        return misc
    pieces = misc.split("|")
    pieces = [x for x in pieces if x.split("=", maxsplit=1)[0] not in ("SpaceAfter", "SpacesAfter", "SpacesBefore")]
    if len(pieces) > 0:
        return "|".join(pieces)
    return None

def substitute_space_misc(misc, space_misc):
    space_misc_pieces = space_misc.split("|") if space_misc else []
    space_misc_after = None
    space_misc_before = None
    for piece in space_misc_pieces:
        if piece.startswith("SpaceBefore"):
            space_misc_before = piece
        elif piece.startswith("SpaceAfter") or piece.startswith("SpacesAfter"):
            space_misc_after = piece
        else:
            raise AssertionError("An unknown piece wound up in the misc space fields: %s" % piece)

    pieces = misc.split("|")
    new_pieces = []
    for piece in pieces:
        if piece.startswith("SpaceBefore"):
            if space_misc_before:
                new_pieces.append(space_misc_before)
                space_misc_before = None
        elif piece.startswith("SpaceAfter") or piece.startswith("SpacesAfter"):
            if space_misc_after:
                new_pieces.append(space_misc_after)
                space_misc_after = None
        else:
            new_pieces.append(piece)
    if space_misc_after:
        new_pieces.append(space_misc_after)
    if space_misc_before:
        new_pieces.append(space_misc_before)
    if len(new_pieces) == 0:
        return None
    return "|".join(new_pieces)

class JavaProtobufContext(object):
    """
    A generic context for sending requests to a java program using protobufs in a subprocess
    """
    def __init__(self, classpath, build_response, java_main, extra_args=None):
        self.classpath = resolve_classpath(classpath)
        self.build_response = build_response
        self.java_main = java_main

        if extra_args is None:
            extra_args = []
        self.extra_args = extra_args
        self.pipe = None

    def open_pipe(self):
        self.pipe = subprocess.Popen(["java", "-cp", self.classpath, self.java_main, "-multiple"] + self.extra_args,
                                     stdin=subprocess.PIPE,
                                     stdout=subprocess.PIPE)

    def close_pipe(self):
        if self.pipe.poll() is None:
            self.pipe.stdin.write((0).to_bytes(4, 'big'))
            self.pipe.stdin.flush()
            self.pipe = None

    def __enter__(self):
        self.open_pipe()
        return self

    def __exit__(self, type, value, traceback):
        self.close_pipe()

    def process_request(self, request):
        if self.pipe is None:
            raise RuntimeError("Pipe to java process is not open or was closed")

        text = request.SerializeToString()
        self.pipe.stdin.write(len(text).to_bytes(4, 'big'))
        self.pipe.stdin.write(text)
        self.pipe.stdin.flush()
        response_length = self.pipe.stdout.read(4)
        if len(response_length) < 4:
            raise BrokenPipeError("Could not communicate with java process!")
        response_length = int.from_bytes(response_length, "big")
        response_text = self.pipe.stdout.read(response_length)
        response = self.build_response()
        response.ParseFromString(response_text)
        return response




"""
Processor for performing part-of-speech tagging
"""

import torch

from stanza.models.common import doc
from stanza.models.common.utils import unsort
from stanza.models.common.vocab import VOCAB_PREFIX, CompositeVocab
from stanza.models.pos.data import Dataset
from stanza.models.pos.trainer import Trainer
from stanza.pipeline._constants import *
from stanza.pipeline.processor import UDProcessor, register_processor
from stanza.utils.get_tqdm import get_tqdm

tqdm = get_tqdm()

@register_processor(name=POS)
class POSProcessor(UDProcessor):

    # set of processor requirements this processor fulfills
    PROVIDES_DEFAULT = set([POS])
    # set of processor requirements for this processor
    REQUIRES_DEFAULT = set([TOKENIZE])

    def _set_up_model(self, config, pipeline, device):
        # get pretrained word vectors
        self._pretrain = pipeline.foundation_cache.load_pretrain(config['pretrain_path']) if 'pretrain_path' in config else None
        args = {'charlm_forward_file': config.get('forward_charlm_path', None),
                'charlm_backward_file': config.get('backward_charlm_path', None)}
        # set up trainer
        self._trainer = Trainer(pretrain=self.pretrain, model_file=config['model_path'], device=device, args=args, foundation_cache=pipeline.foundation_cache)
        self._tqdm = 'tqdm' in config and config['tqdm']

    def __str__(self):
        return "POSProcessor(%s)" % self.config['model_path']

    def get_known_xpos(self):
        """
        Returns the xpos tags known by this model
        """
        if isinstance(self.vocab['xpos'], CompositeVocab):
            if len(self.vocab['xpos']) == 1:
                return [k for k in self.vocab['xpos'][0]._unit2id.keys() if k not in VOCAB_PREFIX]
            else:
                return {k: v.keys() - VOCAB_PREFIX for k, v in self.vocab['xpos']._unit2id.items()}
        return [k for k in self.vocab['xpos']._unit2id.keys() if k not in VOCAB_PREFIX]

    def is_composite_xpos(self):
        """
        Returns if the xpos tags are part of a composite vocab
        """
        return isinstance(self.vocab['xpos'], CompositeVocab)

    def get_known_upos(self):
        """
        Returns the upos tags known by this model
        """
        keys = [k for k in self.vocab['upos']._unit2id.keys() if k not in VOCAB_PREFIX]
        return keys

    def get_known_feats(self):
        """
        Returns the features known by this model
        """
        values = {k: v.keys() - VOCAB_PREFIX for k, v in self.vocab['feats']._unit2id.items()}
        return values

    def process(self, document):
        # currently, POS models are saved w/o the batch_maximum_tokens flag
        maximum_tokens = self.config.get('batch_maximum_tokens', 5000)

        dataset = Dataset(
            document, self.config, self.pretrain, vocab=self.vocab, evaluation=True,
            sort_during_eval=True)
        batch = iter(dataset.to_length_limited_loader(batch_size=self.config['batch_size'], maximum_tokens=maximum_tokens))
        preds = []

        idx = []
        with torch.no_grad():
            if self._tqdm:
                batch = tqdm(batch)
            for i, b in enumerate(batch):
                idx.extend(b[-1])
                preds += self.trainer.predict(b)

        preds = unsort(preds, idx)
        dataset.doc.set([doc.UPOS, doc.XPOS, doc.FEATS], [y for x in preds for y in x])
        return dataset.doc



"""
Test some use cases of the chuliu_edmonds algorithm

(currently just the tarjan implementation)
"""

import numpy as np
import pytest

from stanza.models.common.chuliu_edmonds import tarjan

pytestmark = [pytest.mark.travis, pytest.mark.pipeline]

def test_tarjan_basic():
    simple = np.array([0, 4, 4, 4, 0])
    result = tarjan(simple)
    assert result == []

    simple = np.array([0, 2, 0, 4, 2, 2])
    result = tarjan(simple)
    assert result == []

def test_tarjan_cycle():
    cycle_graph = np.array([0, 3, 1, 2])
    result = tarjan(cycle_graph)
    expected = np.array([False,  True,  True,  True])
    assert len(result) == 1
    np.testing.assert_array_equal(result[0], expected)

    cycle_graph = np.array([0, 3, 1, 2, 5, 6, 4])
    result = tarjan(cycle_graph)
    assert len(result) == 2
    expected = [np.array([False,  True,  True,  True, False, False, False]),
                np.array([False, False, False, False,  True,  True,  True])]
    for r, e in zip(result, expected):
        np.testing.assert_array_equal(r, e)



import pytest

from stanza.models.constituency import in_order_compound_oracle
from stanza.models.constituency import tree_reader
from stanza.models.constituency.parse_transitions import CloseConstituent, OpenConstituent, Shift, TransitionScheme
from stanza.models.constituency.transition_sequence import build_treebank

from stanza.tests.constituency.test_transition_sequence import reconstruct_tree

pytestmark = [pytest.mark.pipeline, pytest.mark.travis]

# A sample tree from PTB with a triple unary transition (at a location other than root)
# Here we test the incorrect closing of various brackets
TRIPLE_UNARY_START_TREE = """
( (S
    (PRN
      (S
        (NP-SBJ (-NONE- *) )
        (VP (VB See) )))
    (, ,)
    (NP-SBJ
      (NP (DT the) (JJ other) (NN rule) )
      (PP (IN of)
        (NP (NN thumb) ))
      (PP (IN about)
        (NP (NN ballooning) )))))
"""

TREES = [TRIPLE_UNARY_START_TREE]
TREEBANK = "\n".join(TREES)

ROOT_LABELS = ["ROOT"]

@pytest.fixture(scope="module")
def trees():
    trees = tree_reader.read_trees(TREEBANK)
    trees = [t.prune_none().simplify_labels() for t in trees]
    assert len(trees) == len(TREES)

    return trees

@pytest.fixture(scope="module")
def gold_sequences(trees):
    gold_sequences = build_treebank(trees, TransitionScheme.IN_ORDER_COMPOUND)
    return gold_sequences

def get_repairs(gold_sequence, wrong_transition, repair_fn):
    """
    Use the repair function and the wrong transition to iterate over the gold sequence

    Returns a list of possible repairs, one for each position in the sequence
    Repairs are tuples, (idx, seq)
    """
    repairs = [(idx, repair_fn(gold_transition, wrong_transition, gold_sequence, idx, ROOT_LABELS, None, None))
               for idx, gold_transition in enumerate(gold_sequence)]
    repairs = [x for x in repairs if x[1] is not None]
    return repairs

def test_fix_shift_close():
    trees = tree_reader.read_trees(TRIPLE_UNARY_START_TREE)
    trees = [t.prune_none().simplify_labels() for t in trees]
    assert len(trees) == 1
    tree = trees[0]

    gold_sequences = build_treebank(trees, TransitionScheme.IN_ORDER_COMPOUND)

    # there are three places in this tree where a long bracket (more than 2 subtrees)
    # could theoretically be closed and then reopened
    repairs = get_repairs(gold_sequences[0], CloseConstituent(), in_order_compound_oracle.fix_shift_close_error)
    assert len(repairs) == 3

    expected_trees = ["(ROOT (S (S (PRN (S (VP (VB See)))) (, ,)) (NP (NP (DT the) (JJ other) (NN rule)) (PP (IN of) (NP (NN thumb))) (PP (IN about) (NP (NN ballooning))))))",
                      "(ROOT (S (PRN (S (VP (VB See)))) (, ,) (NP (NP (NP (DT the) (JJ other)) (NN rule)) (PP (IN of) (NP (NN thumb))) (PP (IN about) (NP (NN ballooning))))))",
                      "(ROOT (S (PRN (S (VP (VB See)))) (, ,) (NP (NP (NP (DT the) (JJ other) (NN rule)) (PP (IN of) (NP (NN thumb)))) (PP (IN about) (NP (NN ballooning))))))"]

    for repair, expected in zip(repairs, expected_trees):
        repaired_tree = reconstruct_tree(tree, repair[1], transition_scheme=TransitionScheme.IN_ORDER_COMPOUND)
        assert str(repaired_tree) == expected

def test_fix_open_close():
    trees = tree_reader.read_trees(TRIPLE_UNARY_START_TREE)
    trees = [t.prune_none().simplify_labels() for t in trees]
    assert len(trees) == 1
    tree = trees[0]

    gold_sequences = build_treebank(trees, TransitionScheme.IN_ORDER_COMPOUND)

    repairs = get_repairs(gold_sequences[0], CloseConstituent(), in_order_compound_oracle.fix_open_close_error)
    print("------------------")
    for repair in repairs:
        print(repair)
        repaired_tree = reconstruct_tree(tree, repair[1], transition_scheme=TransitionScheme.IN_ORDER_COMPOUND)
        print("{:P}".format(repaired_tree))



""" Describes ClusterChecker, a class used to retrieve LEA scores.
See aclweb.org/anthology/P16-1060.pdf. """

from typing import Hashable, List, Tuple

from stanza.models.coref.const import EPSILON
import numpy as np

import math
import logging

logger = logging.getLogger('stanza')


class ClusterChecker:
    """ Collects information on gold and predicted clusters across documents.
    Can be used to retrieve weighted LEA-score for them.
    """
    def __init__(self):
        self._lea_precision = 0.0
        self._lea_recall = 0.0
        self._lea_precision_weighting = 0.0
        self._lea_recall_weighting = 0.0
        self._num_preds = 0.0

        # muc
        self._muc_precision = 0.0
        self._muc_recall = 0.0

        # b3
        self._b3_precision = 0.0
        self._b3_recall = 0.0

        # ceafe
        self._ceafe_precision = 0.0
        self._ceafe_recall = 0.0

    @staticmethod
    def _f1(p,r):
        return (p * r) / (p+r + EPSILON) * 2
    
    def add_predictions(self,
                        gold_clusters: List[List[Hashable]],
                        pred_clusters: List[List[Hashable]]):
        """
        Calculates LEA for the document's clusters and stores them to later
        output weighted LEA across documents.

        Returns:
            LEA score for the document as a tuple of (f1, precision, recall)
        """

        # if len(gold_clusters) == 0:
            # breakpoint()

        self._num_preds += 1
        
        recall, r_weight = ClusterChecker._lea(gold_clusters, pred_clusters)
        precision, p_weight = ClusterChecker._lea(pred_clusters, gold_clusters)

        self._muc_recall +=  ClusterChecker._muc(gold_clusters, pred_clusters)
        self._muc_precision += ClusterChecker._muc(pred_clusters, gold_clusters)

        self._b3_recall += ClusterChecker._b3(gold_clusters, pred_clusters)
        self._b3_precision += ClusterChecker._b3(pred_clusters, gold_clusters)

        ceafe_precision, ceafe_recall = ClusterChecker._ceafe(pred_clusters, gold_clusters)
        if math.isnan(ceafe_precision) and len(gold_clusters) > 0:
            # because our model predicted no clusters
            ceafe_precision = 0.0

        self._ceafe_precision += ceafe_precision
        self._ceafe_recall += ceafe_recall

        self._lea_recall += recall
        self._lea_recall_weighting += r_weight
        self._lea_precision += precision
        self._lea_precision_weighting += p_weight

        doc_precision = precision / (p_weight + EPSILON)
        doc_recall = recall / (r_weight + EPSILON)
        doc_f1 = (doc_precision * doc_recall) \
            / (doc_precision + doc_recall + EPSILON) * 2
        return doc_f1, doc_precision, doc_recall

    @property
    def bakeoff(self):
        """ Get the F1 macroaverage score used by the bakeoff """
        return sum(self.mbc)/3

    @property
    def mbc(self):
        """ Get the F1 average score of (muc, b3, ceafe) over docs """
        avg_precisions = [self._muc_precision, self._b3_precision, self._ceafe_precision]
        avg_precisions = [i/(self._num_preds + EPSILON) for i in avg_precisions]

        avg_recalls = [self._muc_recall, self._b3_recall, self._ceafe_recall]
        avg_recalls = [i/(self._num_preds + EPSILON) for i in avg_recalls]

        avg_f1s = [self._f1(p,r) for p,r in zip(avg_precisions, avg_recalls)]

        return avg_f1s

    @property
    def total_lea(self):
        """ Returns weighted LEA for all the documents as
        (f1, precision, recall) """
        precision = self._lea_precision / (self._lea_precision_weighting + EPSILON)
        recall = self._lea_recall / (self._lea_recall_weighting + EPSILON)
        f1 = self._f1(precision, recall)
        return f1, precision, recall

    @staticmethod
    def _lea(key: List[List[Hashable]],
             response: List[List[Hashable]]) -> Tuple[float, float]:
        """ See aclweb.org/anthology/P16-1060.pdf. """
        response_clusters = [set(cluster) for cluster in response]
        response_map = {mention: cluster
                        for cluster in response_clusters
                        for mention in cluster}
        importances = []
        resolutions = []
        for entity in key:
            size = len(entity)
            if size == 1:  # entities of size 1 are not annotated
                continue
            importances.append(size)
            correct_links = 0
            for i in range(size):
                for j in range(i + 1, size):
                    correct_links += int(entity[i]
                                         in response_map.get(entity[j], {}))
            resolutions.append(correct_links / (size * (size - 1) / 2))
        res = sum(imp * res for imp, res in zip(importances, resolutions))
        weight = sum(importances)
        return res, weight

    @staticmethod
    def _muc(key: List[List[Hashable]],
             response: List[List[Hashable]]) -> float:
        """ See aclweb.org/anthology/P16-1060.pdf. """

        response_clusters = [set(cluster) for cluster in response]
        response_map = {mention: cluster
                        for cluster in response_clusters
                        for mention in cluster}

        top = 0 # sum over k of |k_i| - response_partitions(|k_i|)
        bottom = 0 # sum over k of |k_i| - 1

        for entity in key:
            S = len(entity)
            # we need to figure the number of DIFFERENT clusters 
            # the response assigns to members of the entity; ideally
            # this number is 1 (i.e. they are all assigned the same
            # coref).
            response_clusters = [response_map.get(i, None) for i in entity]
            # and dedplicate
            deduped = []
            for i in response_clusters:
                if i == None:
                    deduped.append(i)
                elif i not in deduped:
                    deduped.append(i)
            # the "partitions" will then be size of the deduped list
            p_k = len(deduped)
            top += (S - p_k)
            bottom += (S - 1)
        
        try:
            return top/bottom
        except ZeroDivisionError:
            logger.warning("muc got a zero division error because the model predicted no spans!")
            return 0 # +inf technically

    @staticmethod
    def _b3(key: List[List[Hashable]],
            response: List[List[Hashable]]) -> float:
        """ See aclweb.org/anthology/P16-1060.pdf. """
        
        response_clusters = [set(cluster) for cluster in response]

        top = 0 # sum over key and response of (|k intersect response|^2/|k|)
        bottom = 0 # sum over k of |k_i|

        for entity in key:
            bottom += len(entity)
            entity = set(entity)

            for res_entity in response_clusters:
                top += (len(entity.intersection(res_entity))**2)/len(entity)

        try:
            return top/bottom
        except ZeroDivisionError:
            logger.warning("b3 got a zero division error because the model predicted no spans!")
            return 0 # +inf technically



    @staticmethod
    def _phi4(c1, c2):
        return 2 * len([m for m in c1 if m in c2]) / float(len(c1) + len(c2))

    @staticmethod
    def _ceafe(clusters: List[List[Hashable]], gold_clusters: List[List[Hashable]]):
        """ see https://github.com/ufal/corefud-scorer/blob/main/coval/eval/evaluator.py """

        try:
            from scipy.optimize import linear_sum_assignment
        except ImportError:
            raise ImportError("To perform CEAF scoring, please install scipy via `pip install scipy` for the Kuhn-Munkres linear assignment scheme.")

        clusters = [c for c in clusters]
        scores = np.zeros((len(gold_clusters), len(clusters)))
        for i in range(len(gold_clusters)):
            for j in range(len(clusters)):
                scores[i, j] = ClusterChecker._phi4(gold_clusters[i], clusters[j])
        row_ind, col_ind = linear_sum_assignment(-scores)
        similarity = scores[row_ind, col_ind].sum()

        # precision, recall
        try:
            prec = similarity/len(clusters)
        except ZeroDivisionError:
            logger.warning("ceafe got a zero division error because the model predicted no spans!")
            prec = 0
        recc = similarity/len(gold_clusters)
        return prec, recc




""" Describes Config, a simple namespace for config values.

For description of all config values, refer to config.toml.
"""

from dataclasses import dataclass
from typing import Dict, List


@dataclass
class Config:  # pylint: disable=too-many-instance-attributes, too-few-public-methods
    """ Contains values needed to set up the coreference model. """
    section: str

    # TODO: can either eliminate data_dir or use it for the train/dev/test data
    data_dir: str
    save_dir: str
    save_name: str

    train_data: str
    dev_data: str
    test_data: str

    device: str

    bert_model: str
    bert_window_size: int

    embedding_size: int
    sp_embedding_size: int
    a_scoring_batch_size: int
    hidden_size: int
    n_hidden_layers: int

    max_span_len: int

    rough_k: int

    lora: bool
    lora_alpha: int
    lora_rank: int
    lora_dropout: float

    full_pairwise: bool

    lora_target_modules: List[str]
    lora_modules_to_save: List[str]

    clusters_starts_are_singletons: bool
    bert_finetune: bool
    dropout_rate: float
    learning_rate: float
    bert_learning_rate: float
    # we find that setting this to a small but non-zero number
    # makes the model less likely to forget how to do anything
    bert_finetune_begin_epoch: float
    train_epochs: int
    bce_loss_weight: float

    tokenizer_kwargs: Dict[str, dict]
    conll_log_dir: str

    save_each_checkpoint: bool
    log_norms: bool
    singletons: bool
    
    max_train_len: int
    use_zeros: bool

    lang_lr_attenuation: str
    lang_lr_weights: str



import pytest

import stanza
from stanza.models.common import short_name_to_treebank

pytestmark = [pytest.mark.travis, pytest.mark.pipeline]

def test_short_name():
    assert short_name_to_treebank.short_name_to_treebank("en_ewt") == "UD_English-EWT"

def test_canonical_name():
    assert short_name_to_treebank.canonical_treebank_name("UD_URDU-UDTB") == "UD_Urdu-UDTB"
    assert short_name_to_treebank.canonical_treebank_name("ur_udtb") == "UD_Urdu-UDTB"
    assert short_name_to_treebank.canonical_treebank_name("Unban_Mox_Opal") == "Unban_Mox_Opal"
